/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <contacts.h>

#include "WAppWindow.h"
#include "WDebugBase.h"

#include "CtListView.h"
#include "CtVcardApp.h"

CtVcardApp::CtVcardApp()
	: m_Naviframe(NULL)
{
	int err = contacts_connect();
	if (err != CONTACTS_ERROR_NONE) {
		WDEBUG("contacts_connect failed: %d", err);
	}
}

CtVcardApp::~CtVcardApp()
{
	contacts_disconnect();
}

bool CtVcardApp::onCreate()
{
	ContactsCommon::loadLocalization();
	attachWindow(new WAppWindow("Import", ELM_WIN_BASIC));

	m_Naviframe = new WNaviframe();
	m_Naviframe->setOnLastItemPop([](bool *popOut) {
		ui_app_exit();
	});
	getWindow()->attachBaseUiObject(m_Naviframe);

	return true;
}

void CtVcardApp::onAppControl(app_control_h request, bool firstLaunch)
{
	WHIT();
	CtListView *view = NULL;
	char *operation = NULL;

	app_control_get_operation(request, &operation);

	if (operation) {
		if (strcmp(operation, APP_CONTROL_OPERATION_VIEW) == 0) {
			char *uri = NULL;
			app_control_get_uri(request, &uri);

			if (uri) {
				view = new CtListView(uri);
				free(uri);
			}
		}

		free(operation);
	}

	if (view) {
		m_Naviframe->push(view);
	}  else {
		app_control_h reply;
		app_control_create(&reply);
		app_control_reply_to_launch_request(reply, request, APP_CONTROL_RESULT_FAILED);
		app_control_destroy(reply);
		ui_app_exit();
	}
}
