#!/bin/bash

### How to add in .gbs.conf file ############################################################################
##   In order to use this script, please add profile as follows
##   
##   [general]
##   #Current profile name which should match a profile section name
##   profile = profile.slp
##   
##   [profile.slp]
##   repos = repo.Z130H_SWA_DD
##   
##   # Kiran SM-Z130H_SWA_DD (3G)
##   [profile.Z130H_SWA_DD]
##   repos = repo.Z130H_SWA_DD
##   [repo.Z130H_SWA_DD]
##   url = http://10.252.248.23:8008/snapshots/kiran/SM-Z130H_SWA_DD_NEWBD/release/RelZ130HDD-NEWBD_20140929.003/
### How to add in .gbs.conf file ############################################################################

YELLOWGREEN="\e[103;32m"
YELLOWRED="\e[103;31m"
GREEN="\e[0;32m"
RED="\e[0;31m"
ENDCOL="\e[m"

#######
#ver1.1
#######
PROGRAM_NAME="build.sh"
GBSROOT=$(grep "gbsroot = " ~/.gbs.conf | awk '{ print $3 }')
if [[ $GBSROOT == "" ]];then
GBSROOT="$HOME/GBS-ROOT"
fi
DEFINES=""
FLAGS=""
INSTALL=""
TEMP=$(getopt -n $PROGRAM_NAME -o mincd:Cqwlxh --long help,inst,install,incremental,noinit,ccache,define:,clean,quite,wear,lite -- "$@")

eval set -- "$TEMP"
while true; do
        case $1 in
                -C|--ccache)
                        CCACHE=1; shift; continue
                ;;
                -c|--clean)
                        CLEAN=1; shift; continue
                ;;
                -m|--incremental)
                        INCREMENTAL=1; shift; continue
                ;;
                -h|--help)
echo "-c : gbs build --clean"
echo "-n : gbs build --noinit"
echo "-m : gbs build --incremental"
echo "-x : git clean -xdf"
echo "-q : gbs build | grep error (print out only error messages)"
echo "-i : gbs build, then run install.sh"
echo ""
echo "install.sh : install without debug packages"
echo "install.sh debug : install with debug packages"

exit 0
                ;;
                -n|--noinit)
                        NOINIT=1; shift; continue
                ;;
                -q|--quite)
                        QUITE=1; shift; continue
                ;;
                -d|--define)
                        DEFINES="$DEFINES --define \"$2\""; shift; shift; continue
                ;;
                -w|--wear)
                        WEAR=1; shift; continue
                ;;
                -R|--release)
                        RELEASE=1; shift; continue
                ;;
                -l|--lite)
                        LITE=1; shift; continue
                ;;
                -i|--inst|--install)
                        INSTALL=1; shift; continue
                ;;
                -x)
                        GITCLEAN=1; shift; continue
                ;;
                --)
                        # no more arguments to parse
                        break
                ;;
                *)
                        printf "Unknown option %s\n" "$1"
                        exit 1
                ;;
        esac
done

##### FORCED option #####
KIRAN=1;
#####################

if [[ $GITCLEAN == 1 ]];then
git clean -xdf
fi

if [[ $WEAR != 1 ]];then
	if [[ $KIRAN == 1 ]];then
	MODEL="Z130H_SWA_DD"
	else
	MODEL="open"
	fi
else
MODEL="tizenw"
fi

if [[ $NOINIT == 1 ]];then
FLAGS="$FLAGS --noinit"
fi
if [[ $INCREMENTAL == 1 ]];then
FLAGS="$FLAGS --incremental"
fi
if [[ $CCACHE == 1 ]];then
FLAGS="$FLAGS --incremental --ccache"
fi
if [[ $CLEAN == 1 ]];then
FLAGS="$FLAGS --clean"
fi
#if [[ $RELEASE != 1 ]];then
#DEFINES=$DEFINES' --define "sec_build_binary_debug_enable 1"'
#fi

echo "====================================================="
echo "GBSROOT: $GBSROOT"
echo "Model: $MODEL"
echo "FLAGS  : $FLAGS"
echo "DEFINES:$DEFINES"
echo "================================= enter to continue ="
CMD="gbs build -A armv7l -P $MODEL -B $GBSROOT/$MODEL --include-all --keep-packs $FLAGS $DEFINES"

echo $CMD

if [[ $QUITE == 1 ]];then
$CMD | grep error -i
else
$CMD | tee build.txt
fi


### Warning check ############################################################################
echo -e '\E[0;31m'"\033[1m  Checking... waiting...\n\n \033[0m"

sed -n '/warning/p' build.txt > build_1.txt
sed -n '/contacts/p' build_1.txt > build_2.txt
sed '/edc/d' build_2.txt > build_3.txt
sed 's/:.*$//' build_3.txt > build_4.txt
sed 's/^.*\///' build_4.txt > build_5.txt
sort -u build_5.txt > build_6.txt

#echo $except
result=0
while read line;
do
    if [[ $except != *$line* ]]; then
        result=1
        echo -e '\E[40;32m'"\033[1m  Found warnings you have just created in $line \033[0m"
        echo -e '\E[40;32m'"\033[1m  Check below files...\033[0m"
        grep -rn $line build_3.txt
    fi
done < build_6.txt

if [ $result = 1 ]; then
    echo -e '\E[40;31m'"\033[1m DO NOT UPLOADING... \033[0m"
    echo -e '\E[40;31m'"\033[1m Please fix warnings before uploading.\033[0m"
	exit 255
else
    echo -e '\E[40;37m'"\033[1m No warnings are detected. continue your job.\033[0m"
fi

rm build.txt
rm build_1.txt
rm build_2.txt
rm build_3.txt
rm build_4.txt
rm build_5.txt
rm build_6.txt
### Warning check ############################################################################


if [[ $INSTALL == 1 ]];then
./install.sh
fi

exit 0

