/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __CT_STRING_H__
#define __CT_STRING_H__

#include <app.h>
#include <Elementary.h>
#include <contacts.h>
#include <string>

#include "CtString.h"

#ifdef __cplusplus
extern "C"
{
#endif

#include <libintl.h>

enum {
	PHTEXT_KEYPAD,
	PHTEXT_FAVORITES,
	PHTEXT_SPEEDDIAL,
	PHTEXT_VOICE_MAIL,
	PHTEXT_LOGS,
	PHTEXT_SPEED_DIAL_REQUEST,
	PHTEXT_EMERGENCY_CALL,
	PHTEXT_NO_LOGS,
	PHTEXT_EXCEED_NUM_MAX_LENGTH,
	PHTEXT_DELETING_LOGS,
	PHTEXT_INITIALIZING,
	PHTEXT_INCOMMING_CALL,
	PHTEXT_OUTGOING_CALL,
	PHTEXT_MISSED_CALL,
	PHTEXT_SENT_MESSAGE,
	PHTEXT_RECEIVED_MESSAGE,
	PHTEXT_REJECTED_CALL_ABB,
	PHTEXT_EDIT_NUMBER_BEFORE_CALL,
	PHTEXT_SOUND_MODE_SET_TO_SOUND_ABB,
	PHTEXT_SOUND_MODE_SET_TO_VIBRATE_ABB,
	PHTEXT_ADD_PD_SEC_PAUSE_ABB,
	PHTEXT_ALL_LOGS,
	PHTEXT_ALL_CALLS_ABB,
	PHTEXT_MISSED_CALLS_ABB,
	PHTEXT_DIALLED_CALLS,
	PHTEXT_RECEIVED_CALLS_ABB,
	PHTEXT_REJECTED_CALLS,
	PHTEXT_ALL_MESSAGES_M_SMS,
	PHTEXT_SET_DIAL_TO_START_SCREEN,
	PHTEXT_CALLER_ID_NOTIFICATION,
	PHTEXT_CALLER_ID_NON_NOTIFICATION,
	PHTEXT_CALLER_ID_DISPLAY,
	PHTEXT_CALLER_ID_NOT_DISPLAY,
	PHTEXT_INTERNATIONAL_CALL,
	PHTEXT_SET_START_UP_SCREEN,
	PHTEXT_SET_RECENT_CALLS_TO_START_SCREEN,
	PHTEXT_SEARCH_LOCATION,
	PHTEXT_VOICE_COURIER,
	PHTEXT_VIEW_BY,
	PHTEXT_SPEED_DIAL_SETTING,
	PHTEXT_CONVERT_INTERNATIONAL_PREFIX,
	PHTEXT_IMADOCO_SEARCH,
	PHTEXT_IMADOCO_SIMPLE_SEARCH,
	PHTEXT_CONNECT_TO_THE_SITE,
	PHTEXT_UPDATE_EXISTING,
	PHTEXT_EMERGENCY_CONTACTS,
	PHTEXT_EMERGENCY_SERVICE,
	PHTEXT_MINIMISE,
	PHTEXT_UNAVAILABLE,
	PHTEXT_PAYPHONE,
	PHTEXT_REJECT_BY_USER,
	PHTEXT_CONVERT_COUNTRY_CODE,
	PHTEXT_SET_VOICEMAIL,
	PHTEXT_PLAY_VOICE_MAIL,
	PHTEXT_RECORD_KOE_TAKU,
	PHTEXT_PLAY_KOE_TAKU,
	PHTEXT_KOE_TAKU_LIST,
	PHTEXT_KOE_TAKU_SETTING,
	PHTEXT_START_VOICE_MAIL,
	PHTEXT_STOP_VOICE_MAIL,
	PHTEXT_SET_VOICE_MAIL,
	PHTEXT_RING_TIME,
	PHTEXT_START_CALL_FWD,
	PHTEXT_STOP_CALL_FWD,
	PHTEXT_SET_CALL_FWD,
	PHTEXT_CHECK_SETTING,
	PHTEXT_CHECK_OPERATION,
	PHTEXT_ROAMING_GUIDE,
	PHTEXT_INCOMING_SETTING,
	PHTEXT_CALL_SETTING,
	PHTEXT_MISSING_VOICEMAIL_NUMBER,
	PHTEXT_ADD_NUMBER,
	PHTEXT_PIN1_CHANGED,
	PHTEXT_PIN2_CHANGED,
	PHTEXT_PASSWORDS_DO_NOT_MATCH,
	PHTEXT_PUK1_VERIFIED,
	PHTEXT_PUK2_VERIFIED,
	PHTEXT_PIN1_BLOCKED,
	PHTEXT_PIN2_BLOCKED,
	PHTEXT_INCORRECT_PUK1_CODE,
	PHTEXT_INCORRECT_PUK2_CODE,
	PHTEXT_YOUR_PIN_MUST_BE_BETWEEN_P1SD_AND_P2SD_DIGITS_LONG_TRY_AGAIN,
	PHTEXT_INCORRECT_PIN1,
	PHTEXT_INCORRECT_PIN2,
	PHTEXT_CONNECTION_PROBLEM_OR_INVALID_MMI_CODE,
	PHTEXT_SELECT_ITEM,
	PHTEXT_SIM_CARD_BUSY,
	PHTEXT_SIM_CARD_ERROR,
	PHTEXT_SIM_BLOCKED,
	PHTEXT_SIM_RUIM_CARD_LOCKED,
	PHTEXT_SIM_CARD_INITIALISING_ING,
	PHTEXT_NO_SIM_CARD_FOUND_PLEASE_INSERT_A_SIM_CARD_INTO_THE_PHONE,
	PHTEXT_REJECTION_SETTINGS_ABB,
	PHTEXT_VIEW_CONTACT_ABB,
	PHTEXT_LOGENTRYUNKNOWN,
	PHTEXT_NO_CALLS,
	PHTEXT_NO_MISSED_CALLS,
	PHTEXT_NO_DIALLED_CALLS,
	PHTEXT_NO_RECEIVED_CALLS,
	PHTEXT_NO_REJECTED_CALLS,
	PHTEXT_NO_MESSAGES,
	PHTEXT_CALL_DURATION,
	PHTEXT_COPY_TO_DIALLING_SCREEN,
	PHTEXT_DIALINPUT_SEND_NUMBER_TMO,
	PHTEXT_SENT_MESSAGES,
	PHTEXT_RECEIVED_MESSAGES,
	PHTEXT_SELECT_SPEED_DIAL_CONTACT_ABB,
	PHTEXT_VOICE_CALL,
	PHTEXT_ALL_CALLS,
	PHTEXT_RECEIVED_CALLS,
	PHTEXT_LAST_CALL,
	PHTEXT_VIEW_MORE,
	PHTEXT_SEARCH_RESULT,
	PHTEXT_ADDED_TO_REJECT_LIST,
	PHTEXT_REMOVED_FROM_REJECT_LIST,
	PHTEXT_UNKNOWN_M_NUMBER,
	PHTEXT_RESET,
	PHTEXT_MANAGE_REJECT_LIST_IN_CALL_SETTINGS,
	PHTEXT_DO_NOT_SHOW_AGAIN,
	PHTEXT_ASSIGN,
	PHTEXT_ON_ABB,
	PHTEXT_OFF_ABB_M_STATUS,
	PHTEXT_SPEED_DIAL_SETTINGS_ABB,
	PHTEXT_SWAP_SPEED_DIAL_NUMBER_ABB,
	PHTEXT_VIEW,
	PHTEXT_ENTER_NUMBER,
	PHTEXT_SETTING_COMPLETED,
	PHTEXT_NO_SPEED_DIAL_NUMBER_ASSIGNED_ASSIGN_NOW_Q,
	PHTEXT_SWIPE_TO_CALL_SEND_MESSAGES_ABB,
	PHTEXT_UNASSIGNED_M_STATUS_ABB,
	PHTEXT_EMERGENCY_SERVICES_NUMBER_ABB,
	PHTEXT_COUNTRY_CODES_ABB,
	PHTEXT_DOCOMO_WORDING_SEARCH,
	PHTEXT_DOCOMO_WORDING_UPDATE_EXISTING,
	PHTEXT_DOCOMO_WORDING_CREATE_CONTACT,
	PHTEXT_EMERGENCY_CALL_Q,
	PHTEXT_ENTER_NUMBER_ABB,
	PHTEXT_REMOVED,
	PHTEXT_COUNTRY_CODE_NEED,
	PHTEXT_REMOVE,
	PHTEXT_PRIVATE_NUMBER,
	PHTEXT_USE_TOUCH_TONE_KEYPAD,
	PHTEXT_RETURN_TO_CALL_IN_PROGRESS,
	PHTEXT_ADD_CALL,
	PHTEXT_MINI_APPS_CANNOT_BE_OPENED_WHILE_SCREEN_READER_IS_ENABLED_SCREEN_READER_WILL_BE_DISABLED,
	PHTEXT_CLOSE_APPLICATION_Q,
	PHTEXT_MAXIMUM_NUMBER_OF_REJECT_LISTS_REACHED,
	PHTEXT_UNABLE_TO_OPEN_PHONE_IN_MINI_MODE_DURING_CALL_ABB,
	PHTEXT_HASH,
	PHTEXT_ASTERISK,
	PHTEXT_CREATE_CONTACT,
	PHTEXT_SPEED_DIAL_ASSIGNED,
	PHTEXT_REMOVE_FROM_EMERGENCY_CONTACT_SHORTCUTS_Q,
	PHTEXT_UNSAVED_M_NOUN,
	PHTEXT_LOGS_HEADER_DELETE_LOG_ITEMS_ABB,
	PHTEXT_LOGS_POP_DELETE_THIS_LOG_ITEM_Q,
	PHTEXT_LOGS_POP_DELETE_PD_LOG_ITEMS_Q,
	PHTEXT_DELETE_LOG_ABB,
	PHTEXT_CLOSE_APPLICATION_ABB,
	PHTEXT_NO_SPEED_DIAL_ENTRY
};	//PH_GET_TEXT_BASIC

enum {
	PHTEXT_OPEN_PS_PHONE,
	PHTEXT_ENTER_THE_NUMBER_YOU_WANT_TO_DIAL,
	PHTEXT_TAP_PS_TO_MAKE_A_VOICE_CALL,
	PHTEXT_TAP_END_CALL,
	PHTEXT_TAP_PS_TO_MAKE_A_VIDEO_CALL,
	PHTEXT_NOW_YOU_KNOW_HOW_TO_MAKE_CALLS,
	PHTEXT_ENTER_THE_NUMBER_YOU_WANT_TO_SAVE,
	PHTEXT_TAP_ADD_TO_CONTACTS,
	PHTEXT_ENTER_THE_CONTACT_DETAILS_AND_THEN_TAP_SAVE,
	PHTEXT_THE_CONTACT_HAS_BEEN_SAVED,
	PHTEXT_SELECT_A_CONTACT_FROM_THE_LIST,
	PHTEXT_SELECT_A_SAVING_OPTION,
}; //PH_GET_TEXT_HELP

enum {
	CTTEXT_ALL_CONTACTS,
	CTTEXT_NOT_ASSIGNED,
	CTTEXT_SDN,
	CTTEXT_PHONE,
	CTTEXT_GROUP,
	CTTEXT_CREATE_CONTACT,
	CTTEXT_DEFAULT,
	CTTEXT_WEBSITE,
	CTTEXT_IM,
	CTTEXT_EMAIL,
	CTTEXT_NUMBER,
	CTTEXT_NOTE,
	CTTEXT_POSTAL_ADDRESS,
	CTTEXT_DETAILS,
	CTTEXT_COMPANY,
	CTTEXT_SORT_BY,
	CTTEXT_FIRSTNAME,
	CTTEXT_LASTNAME,
	CTTEXT_MIDDLE,
	CTTEXT_SUFFIX,
	CTTEXT_NICKNAME,
	CTTEXT_SELECT,
	CTTEXT_POSTAL_CODE,
	CTTEXT_COUNTRY,
	CTTEXT_CITY,
	CTTEXT_STREET,
	CTTEXT_EXPORTED,
	CTTEXT_DELETING_CONTACTS,
	CTTEXT_REMOVING_CONTACT,
	CTTEXT_IMPORTING_SIM_CONTACTS,
	CTTEXT_IMPORTING_SD_CONTACTS,
	CTTEXT_VCARD_CONTACTS,
	CTTEXT_LOADING_VCARD,
	CTTEXT_IMPORTED,
	CTTEXT_LINKED_CONTACTS,
	CTTEXT_MORE_INFORMATION,
	CTTEXT_JOB_TITLE,
	CTTEXT_DEPARTMENT,
	CTTEXT_GROUP_NAME,
	CTTEXT_ADD_ACCOUNT,
	CTTEXT_ADDING_MEMBERS_TO_GROUP,
	CTTEXT_CONTACTS_TO_DISPLAY,
	CTTEXT_COUNT_CONTACTS,
	CTTEXT_RELATIONSHIP,
	CTTEXT_NO_HISTORIES,
	CTTEXT_ADDING_ING,
	CTTEXT_CUSTOM,
	CTTEXT_TEMP_CUSTOM,
	CTTEXT_INPUT_TEXT,
	CTTEXT_ADDITIONAL_INFO,
	CTTEXT_PO_BOX,
	CTTEXT_STATE,
	CTTEXT_FRIENDS,
	CTTEXT_FAMILY,
	CTTEXT_CO_WORKERS,
	CTTEXT_EMERGENCY_CONTACTS,
	CTTEXT_VIBRATIONS,
	CTTEXT_SET_AS_DEFAULT,
	CTTEXT_EVENT,
	CTTEXT_1_CONTACT,
	CTTEXT_WI_FI_DIRECT,
	CTTEXT_EXPORT_TO_SIM_CARD,
	CTTEXT_ACTIVITIES,
	CTTEXT_GROUP_MESSAGE,
	CTTEXT_GROUP_EMAIL,
	CTTEXT_HOURS_AGO,
	CTTEXT_MINUTE_AGO,
	CTTEXT_REMOVE_Q,
	CTTEXT_INSTANT_MESSENGER,
	CTTEXT_GIVEN_NAME_FIRST,
	CTTEXT_FAMILY_NAME_FIRST,
	CTTEXT_DISPLAY_CONTACT,
	CTTEXT_YANDEX_SEARCH,
	CTTEXT_YANDEX_SEARCH_HPD,
	CTTEXT_ONLY_CONTACTS_WITH_PHONES,
	CTTEXT_DISPLAY_NAMES_BY,
	CTTEXT_MOST_FREQUENT_CONTACTS,
	CTTEXT_PHONETIC_LAST_NAME,
	CTTEXT_PHONETIC_MIDDLE_NAME,
	CTTEXT_PHONETIC_FIRST_NAME,
	CTTEXT_PHONETIC_NAME,
	CTTEXT_MAPS,
	CTTEXT_EXPORT,
	CTTEXT_IMPORT,
	CTTEXT_DOCOMO_PHONEBOOK_JPN_DCM,
	CTTEXT_MY_NUMBER_JPN_DCM,
	CTTEXT_NTT_DOCOMO_INC_JPN_DCM,
	CTTEXT_NUMBER_OF_REGISTERED_DATA_IN_PHONEBOOK_JPN_DCM,
	CTTEXT_PDDATA_JPN_DCM,
	CTTEXT_UNABLE_TO_DO_THIS_ACTION_NOT_ENOUGH_MEMORY_JPN_DCM,
	CTTEXT_TURNED_INTERNATIONAL_USE_SETTING_OFF_BECAUSE_YOU_MOVED_ABROAD_MSG_JPN_DCM,
	CTTEXT_EDITING_DATA_IS_DISCARDED_BECAUSE_CONTACTS_DATA_IS_ALTERED_JPN_DCM,
	CTTEXT_MY_PROFILE_JPN_DCM,
	CTTEXT_VERSION, //not use
	CTTEXT_APPLICATION_NAME,
	CTTEXT_HELP,
	CTTEXT_CLOUD_MENU,
	CTTEXT_OTHER_SETTINGS_JPN_DCM, //no STMS word.
	CTTEXT_ABOUT_JPN_DCM, //no STMS word.
	CTTEXT_PROVIDER_JPN_DCM, //no STMS word.
	CTTEXT_PREFIX,
	CTTEXT_ADDED_TO_HOME_SCREEN,
	CTTEXT_GROUP_INDEX,
	CTTEXT_UNABLE_TO_SAVE_NO_PHONE_NUMBER_ENTERED_FOR_EMERGENCY_CONTACT,
	CTTEXT_ONLY_CONTACTS_WITH_PHONES_DESCRIPTION_MSG,
	CTTEXT_WELCOME_TO_THE_CONTACTS_DASHBOARD, //UI
	CTTEXT_ADD_YOUR_FAVOURITE_CONTACTS_TO_THE_DASHBOARD,
	CTTEXT_SAVE_GROUP_TO,
	CTTEXT_RECOMMENDED,
	CTTEXT_SET_DEFAULT_NAME,
	CTTEXT_MERGE_WITH_SAMSUNG_ACCOUNT,
	CTTEXT_YOUR_FREQUENT_CONTACTS_WILL_BE_DISPLAYED_ON_THE_DASHBOARD,
	CTTEXT_VCARD_ADDED_TO_CONTACTS,
	CTTEXT_COPY_TO_CLIPBOARD,
	CTTEXT_DEVICE,
	CTTEXT_SD_CARD,
	CTTEXT_TO_SIGN_IN_TO_YOUR_SAMSUNG_ACCOUNT_OR_CREATE_AN_ACCOUNT_TAP_CONTINUE,
	CTTEXT_DISPLAY_CONTACTS_IN_SETTINGS,
	CTTEXT_IMPORT_CONTACTS_FROM,
	CTTEXT_EXPORT_CONTACTS_TO,
	CTTEXT_BODY_NUMBER_M_PHONE,
	CTTEXT_BODY_CALL_T_APPLICATIONSETTINGS,
	CTTEXT_HEADER_SELECT_GROUP,
	CTTEXT_BODY_SERVICE_NUMBERS,
	CTTEXT_HEADER_SELECT_NUMBER,
	CTTEXT_HEADER_SELECT_ITEM,
	CTTEXT_ITAB_RECENT,
	CTTEXT_MESSAGE_ALERT,
	CTTEXT_FILE_TRANSFER,
	CTTEXT_CONTACTS,
	CTTEXT_PB_BODY_CHAT,
	CTTEXT_PB_BODY_GALLERY,
	CTTEXT_COM_BODY_RECORD_VIDEO,
	CTTEXT_COM_BODY_MY_FILES,
	CTTEXT_COM_BODY_CALENDAR,
	CTTEXT_UNSELECT_ALL,
	CTTEXT_BUTTON_T_TTS,
	CTTEXT_EDITFIELD,
	CTTEXT_DOUBLE_TAP_TO_OPEN_THE_LIST_T_TTS,
	CTTEXT_SEND_ALL_NAMECARDS,
	CTTEXT_SEND_INDIVIDUAL_NAMECARDS,
	CTTEXT_EMPTY_SLOT_T_TTS,
	CTTEXT_RECORD_SOUND,
	CTTEXT_UNFAVOURITE_T_TTS,
	CTTEXT_SEND_MESSAGE,
	CTTEXT_SEND_EMAIL,
	CTTEXT_SEND_MESSAGE_EMAIL,
	CTTEXT_ACCOUNT,
	CTTEXT_WHEN_YOU_TRANSFER_HELP_MSG,
	CTTEXT_DELETE_CONTACT,
	CTTEXT_CONTACT_SHARING_SETTINGS,
	CTTEXT_GROUP_AND_GROUP_MEMBERS,
	CTTEXT_GROUP_ONLY,
	CTTEXT_CONTACT_PHOTO,
	CTTEXT_ADD_PHOTO,
	CTTEXT_REMOVE_FROM_FAVOURITES,
	CTTEXT_MERGED,
	CTTEXT_LINKED,
	CTTEXT_CREATE,
	CTTEXT_DISCARD_CHANGES_Q,
	CTTEXT_SAMSUNG_ACCOUNT,
	CTTEXT_TAKE_PICTURE,
	CTTEXT_IMAGE,
	CTTEXT_EMAIL_ADDRESS,
	CTTEXT_PHONE_NUMBER,
	CTTEXT_YOUR_DEVICE_CONTACTS_WILL_BE_MERGED_WITH_CONTACTS_FROM_YOUR_SAMSUNG_ACCOUNT,
	CTTEXT_REORDER,
	CTTEXT_AFTER_YOU_CREATE_CONTACTS_CONTACTS_WILL_BE_SHOWN_HERE,
	CTTEXT_CLEAR_ALL,
	CTTEXT_PROCESSING,
	CTTEXT_ADD_CALLER_ID_Q,
	CTTEXT_PICTURES_BY_PEOPLE,
	CTTEXT_IMPORTING_PS_CANCELLED,
	CTTEXT_TAB_LOGS,
	CTTEXT_THIS_GROUP_WILL_BE_DELETED_ONE,
	CTTEXT_THIS_GROUP_WILL_BE_DELETED_MANY,
	CTTEXT_DELETE_GROUP_AND_DELETE_ALL_OF_ITS_MEMBERS_FROM_CONTACTS_Q,
	CTTEXT_DELETE_PD_GROUPS_AND_DELETE_ALL_OF_THEIR_MEMBERS_FROM_CONTACTS_Q,
	CTTEXT_SOME_INFORMATION_MIGHT_BE_LOST_CONTINUE_Q,
	CTTEXT_GROUP_PHOTO,
	CTTEXT_ADDRESS,
	CTTEXT_EXPAND_BUTTON_T_TTS,
	CTTEXT_ADD_ALL_CONTACTS,
	CTTEXT_EDIT_FAVOURITES,
	CTTEXT_MY_CONTACTS,
	CTTEXT_BUSINESS_CARD_RECOGNITION_ABB,
	CTTEXT_PHONE_ABB,
	CTTEXT_DISCARD,
	CTTEXT_ABCDEFGHIJKLMNOPQRSTUVWXYZ,
	CTTEXT_REMOVE_MEMBER,
	CTTEXT_FAVOURITE_CONTACTS,
	CTTEXT_DELETE_CONTACTS,
	CTTEXT_CONTACT_SAVED,
	CTTEXT_DISPLAY_MAP_DCM,
	CTTEXT_ADDRESS_ICONICIER_DCM,
	CTTEXT_MEMO_ICONICIER_DCM,
	CTTEXT_SITE_ICONICIER_DCM,
	CTTEXT_SIM_CARD_DCM,
	CTTEXT_CLOUD_DCM,
	CTTEXT_APROACH_ANNOUNCEMENT,
	CTTEXT_NEW_CONTACT_WILL_BE_SAVED_TO_CONTACTS_ON_DEVICE,
	CTTEXT_NO_FAVOURITES,
	CTTEXT_P1SS_IS_DISABLED_IN_THIS_APPLICATION_TO_ENABLE_THIS_FUNCTION_MSG,
	CTTEXT_MAXIMUM_NUMBER_OF_RECIPIENTS_HPD_REACHED,
	CTTEXT_UP_TO_PD_ITEMS_CAN_BE_SELECTED,
	CTTEXT_YOUR_FREQUENTLY_USED_CONTACTS_WILL_BE_DISPLAYED_IN_YOUR_FAVOURITES,
	CTTEXT_REMOVING_MEMBERS_FROM_GROUP_ING,
	CTTEXT_DELETE_GROUP,
	CTTEXT_CONTACTS_IN_PS,
	CTTEXT_MY_PROFILE_SAVED,
	CTTEXT_EXPORTED_TO_PS,
	CTTEXT_FREQUENTLY_USED_CONTACTS_CANNOT_BE_REORDERED,
	CTTEXT_FREQUENT_CONTACTS_ABB,
	CTTEXT_PAGE_PD_OF_PD,
	CTTEXT_TTS_RADIO_BUTTON,
	CTTEXT_TTS_SELECTED,
	CTTEXT_TTS_DESELECTED,
	CTTEXT_BODY_DAY,
	CTTEXT_BODY_MONTH,
	CTTEXT_BODY_YEAR,
	CTTEXT_TAB_T_TTS,
	CTTEXT_SIM_1,
	CTTEXT_SIM_2,
	CTTEXT_REMOVE_FROM_PS_Q,
	CTTEXT_DOUBLE_TAP_TO_ADD_IMAGE,
	CTTEXT_AFTER_YOU_ADD_CONTACTS_TO_FAVOURITES_THEY_WILL_BE_SHOWN_HERE,
	CTTEXT_GRID_VIEW,
	CTTEXT_LIST_VIEW,
	CTTEXT_SWIPE_TO_CALL_SEND_MSGS,
	CTTEXT_SERVER_IS_NOT_RESPONDING,
	CTTEXT_NETWORK_ERROR,
	CTTEXT_TAP_TO_ADD_A_NEW_BUSINESS_CARD,
	CTTEXT_AFTER_YOU_ADD_BUSINESS_CARDS_THEY_WILL_BE_SHOWN_HERE,
	CTTEXT_CHANGE_CONTACT_PICTURE,
	CTTEXT_DATE_OLDEST,
	CTTEXT_DATE_MOST_RECENT,
	CTTEXT_CHANGING_THE_CONTACT_PICTURE_WILL_DELETE_THE_IMAGE_OF_THE_SCANNED_BUSINESS_CARD,
	CTTEXT_NO_BUSINESS_CARDS,
	CTTEXT_BUSINESS_CARDS,
	CTTEXT_SORT,
	CTTEXT_EXPORTING_CONTACTS,
	CTTEXT_EXPORTING_CONTACTS_ING,
	CTTEXT_PRIMARY_CONTACTS,
	CTTEXT_CREATE_PRIMARY_CONTACT_ABB,
	CTTEXT_SELECT_FROM_CONTACTS,
	CTTEXT_DELETE_THIS_CONTACT_Q,
	CTTEXT_DELETE_PD_GROUPS_Q,
	CTTEXT_DELETE_THIS_GROUP_Q,
	CTTEXT_SELECT_PD_OR_MORE_CONTACTS_TO_LINK,
	CTTEXT_UNABLE_TO_ADD_CONTACTS,
	CTTEXT_ADD_TO_FAVOURITES_INDICATOR,
	CTTEXT_REMOVE_FROM_FAVOURITES_INDICATOR,
	CTTEXT_DELETE_BUSINESS_CARDS,
	CTTEXT_DELETE_BUSINESS_CARD,
	CTTEXT_DELETE_BUSINESS_CARD_Q,
	CTTEXT_ON_OFF_BUTTON,
	CTTEXT_NOTIFICATION,
	CTTEXT_DELETE_GROUPS,
	CTTEXT_UNABLE_TO_ADD_MORE_ITEMS,
	CTTEXT_UNABLE_TO_SELECT_MORE_ITEMS_ABB,
	CTTEXT_DELETING_THESE_BUSINESS_CARDS,
	CTTEXT_DELETING_THIS_BUSINESS_CARD,
	CTTEXT_UNKNOWN,
	CTTEXT_SETTINGS,
}; //CT_GET_TEXT_BASIC
enum {
	CTTEXT_TAP_ADD_TO_CONTACTS,
	CTTEXT_ENTER_THE_CONTACT_DETAILS_AND_THEN_TAP_SAVE,
	CTTEXT_THE_CONTACT_HAS_BEEN_SAVED,
	CTTEXT_SELECT_A_CONTACT_FROM_THE_LIST,
	CTTEXT_TAP_TO_CREATE_A_NEW_CONTACT,
	CTTEXT_AFTER_YOU_MAKE_RECEIVE_CALLS_AND_SEND_RECEIVE_MESSAGES_OR_EMAILS_THE_LOGS_WILL_BE_SHOWN_HERE,
	CTTEXT_SELECT_THE_CONTACTS_YOU_WANT_TO_ADD,
	CTTEXT_TAP_ADD,
	CTTEXT_TAP_ON_THE_AREA_INDICATED,
	CTTEXT_ENTER_THE_CONTACT_DETAILS_THEN_TAP_SAVE,
	CTTEXT_CONTACT_SAVED_HELP,
	CTTEXT_FLICK_LEFT_OR_RIGHT_TO_CHANGE_PAGE,
	CTTEXT_FLICK_UP_TO_SHRINK,
	CTTEXT_FLICK_DOWN_TO_EXPAND,
	CTTEXT_TAP_AND_HOLD_TO_ENTER_EDIT_MODE,
	CTTEXT_TUTORIAL_COMPLETE,
	CTTEXT_ADD_CONTACTS_TO_FAVOURITES,
	CTTEXT_YOU_HAVE_SUCCESSFULLY_ADDED_THE_CONTACT_TO_YOUR_FAVOURITES,
	CTTEXT_THE_LAST_SIZE_CHOSEN_WILL_BE_USED_IN_FAVOURITES,
	CTTEXT_TAP_AND_HOLD_FAVOURITES_TO_REORDER_THEM,
	CTTEXT_EDITING_FAVOURITES_ABB,
	CTTEXT_TAP_ANYWHERE_OUTSIDE_OF_THE_FAVOURITES_AREA_TO_EXIT_EDIT_MODE,
	CTTEXT_SELECT_PS,
	CTTEXT_TAP_TO_REMOVE_A_FAVOURITE,
	CTTEXT_FAVOURITES_IS_ONLY_AVAILABLE_IN_PORTRAIT_MODE,
	CTTEXT_PRESS_THE_MENU_KEY,
	CTTEXT_ADD_TO_FAVOURITES_HELP,
	CTTEXT_PRESS_THE_MENU_KEY_THEN_SELECT_EDIT,
}; //CT_GET_TEXT_HELP

enum {
	CTTEXT_INVALID_EMAIL,
	CTTEXT_SAVE_FAIL,
	CTTEXT_ERR_DB_LOCKED,
	CTTEXT_ALREADY_EXIST,
	CTTEXT_NO_GROUP,
	CTTEXT_NO_NUMBER,
	CTTEXT_NO_EMAIL,
	CTTEXT_NO_CONTACTS,
	CTTEXT_IMPORT_SIM_NO_DATA,
	CTTEXT_EXCEED_LIMIT,
	CTTEXT_SIM_MEMORY_FULL,
	CTTEXT_NO_CONTACTS_HELP_TEXT,
	CTTEXT_NO_HISTORIES_HELP_TEXT,
	CTTEXT_UNABLE_TO_SEND_NAME_CARDS,
	CTTEXT_NONE,
	CTTEXT_NO_RESULTS_FOUND,
	CTTEXT_NO_CONTACTS_GROUP_HELP_TEXT,
};//CT_GET_TEXT_ERR

enum {
	PHTEXT_SELECT_LOGS,
	PHTEXT_SELECT_SPEED_DIAL_CONTACTS,
	PHTEXT_UNBLOCK,
	PHTEXT_CHANGE_CONTACT,
	PHTEXT_ADD_PAUSE,
	PHTEXT_ADD_WAIT,
	PHTEXT_EXIT,
};	// PH_GET_TEXT_BASE_OP

enum {
	PHTEXT_BODY_SUNDAY,
	PHTEXT_BODY_MONDAY,
	PHTEXT_BODY_TUESDAY,
	PHTEXT_BODY_WEDNESDAY,
	PHTEXT_BODY_THURSDAY,
	PHTEXT_BODY_FRIDAY,
	PHTEXT_BODY_SATURDAY,
};	// PH_GET_TEXT_DAY

enum {
	CTTEXT_SET_DEFAULT_NUMBER,
	CTTEXT_SET_DEFAULT_EMAIL,
	CTTEXT_TAKE_A_PHOTO,
	CTTEXT_ADD_TO_HOME,
	CTTEXT_SAVE_CONTACT_TO,
	CTTEXT_SAVE_TO,
	CTTEXT_IMPORT_SIM_CONTACTS,
	CTTEXT_GIVEN_NAME,
	CTTEXT_FAMILY_NAME,
	CTTEXT_SHOW,
	CTTEXT_HIDE,
	CTTEXT_CREATE_GROUP,
	CTTEXT_EXPORT_CONTACTS,
	CTTEXT_IMPORT_CONTACTS,
	CTTEXT_LINK_CONTACT,
	CTTEXT_LINK_CONTACTS,
	CTTEXT_UNLINK_CONTACT,
	CTTEXT_ALL_LINKED_CONTACTS_WILL_BE_DELETED,
	CTTEXT_ADD_TO_FAVOURITES,//UI
	CTTEXT_EDIT_GROUP,//UI
	CTTEXT_ADD_MEMBER,
	CTTEXT_GO_TO_FACEBOOK,
	CTTEXT_SHARE,
	CTTEXT_SAVE_CONTACT_Q,
	CTTEXT_REMOVE,
	CTTEXT_DELETING_SYNC_CONTACT_DELETE_MSG,
	CTTEXT_DELETING_LINKED_SYNC_CONTACT_DELETE_MSG,
	CTTEXT_CONTACT_INCLUDED_READ_ONLY_ACCOUNTS_NOTDELETED_BUT_HIDDEN_MSG,
	CTTEXT_ALL_JOINED_CONTACTS_WILL_BE_DELETED,
	CTTEXT_DELETING_THIS_BUSINESS_CARD_WILL_DELETE_THE_CONTACTS_LINKED_TO_IT_DELETE_Q,
	CTTEXT_DELETE_PD_CONTACTS_Q,
	CTTEXT_DELETE,
};	//CT_GET_TEXT_BASE_OP

enum {
	CTTEXT_REL_NONE,
	CTTEXT_REL_ASSISTANT,//
	CTTEXT_REL_BROTHER,
	CTTEXT_REL_CHILD,
	CTTEXT_REL_DOMESTIC_PARTNER,
	CTTEXT_REL_FATHER,
	CTTEXT_REL_FRIEND,
	CTTEXT_REL_MANAGER,//
	CTTEXT_REL_MOTHER,
	CTTEXT_REL_PARENT,
	CTTEXT_REL_PARTNER,
	CTTEXT_REL_REFERRED_BY,
	CTTEXT_REL_RELATIVE,
	CTTEXT_REL_SISTER,
	CTTEXT_REL_SPOUSE,
	CTTEXT_REL_CUSTOM,
	CTTEXT_REL_ASSISTANT_JPN,
}; //CT_GET_TEXT_RELATIONSHIP_TYPE
#if 0
enum {
	CTTEXT_EMAIL_TYPE_OTHER,
	CTTEXT_EMAIL_TYPE_CUSTOM,
	CTTEXT_EMAIL_TYPE_HOME,
	CTTEXT_EMAIL_TYPE_WORK,
	CTTEXT_EMAIL_TYPE_MOBILE,
};

enum {
	CTTEXT_ADDRESS_TYPE_OTHER,
	CTTEXT_ADDRESS_TYPE_CUSTOM,
	CTTEXT_ADDRESS_TYPE_HOME,
	CTTEXT_ADDRESS_TYPE_WORK,
};

enum {
	CTTEXT_WEBSITE_TYPE_OTHER,
	CTTEXT_WEBSITE_TYPE_CUSTOM,
	CTTEXT_WEBSITE_TYPE_HOME,
	CTTEXT_WEBSITE_TYPE_WORK,
};
#endif

enum {
	PH_GET_TEXT_BASIC,
	PH_GET_TEXT_BASE_OP,
	PH_GET_TEXT_DAY,
	CT_GET_TEXT_BASIC,
	CT_GET_TEXT_ERR,
	CT_GET_TEXT_BASE_OP,
	CT_GET_TEXT_NUM_TYPE,
	CT_GET_TEXT_EMAIL_TYPE,
	CT_GET_TEXT_WEBSITE_TYPE,
	CT_GET_TEXT_POSTAL_TYPE,
	CT_GET_TEXT_EVENT_TYPE,
	CT_GET_TEXT_IM_TYPE,
	CT_GET_TEXT_MONTH,
	CT_GET_TEXT_RELATIONSHIP_TYPE,
	PH_GET_TEXT_HELP,
	CT_GET_TEXT_HELP,
};

#define V_(str) dgettext("contacts", str)

const char* ctGetContactsString(const char* did);

enum {
	PH_SYS_SK_OK,
	CT_SYS_SK_SAVE,
	CT_SYS_SK_DONE,
	PH_SYS_SK_EDIT,
	PH_SYS_SK_DELETE,
	PH_SYS_SK_CANCEL,
	PH_SYS_SK3_BLOCK,
	PH_SYS_SK_NO,
	PH_SYS_SK_YES,
	CT_SYS_SK_ADD,
	CT_SYS_SK_MORE,
	CT_SYS_SK_NEXT,
	PH_SYS_BODY_CALL,
	PH_SYS_BODY_MESSAGE,
	PH_SYS_BODY_VIDEO_CALL,
	PH_SYS_BODY_UNKNOWN,
	PH_SYS_BODY_SELECT_ALL,
	PH_SYS_BODY_CONTACTS,
	PH_SYS_BODY_DETAILS,
	PH_SYS_BODY_TODAY,
	PH_SYS_BODY_YESTERDAY,
	PH_SYS_BODY_PREVIOUS,
	PH_SYS_BODY_NONAME,
	CT_SYS_BODY_CREATE,
	CT_SYS_BODY_MOVE,
	CT_SYS_BODY_COPY,
	CT_SYS_BODY_PHONE,
	CT_SYS_BODY_SEARCH,
	CT_SYS_BODY_BLUETOOTH,
	CT_SYS_BODY_EMAIL,
	CT_SYS_BODY_SIM,
	CT_SYS_BODY_MEMORYCARD,
	CT_SYS_BODY_NAME,
	PH_SYS_BODY_ALL,
	PH_SYS_POP_FAIL,
	PH_SYS_POP_FAILED,
	PH_SYS_POP_CLOSE,
	PH_SYS_POP_REMOVED,
	PH_SYS_POP_DELETED,
	PH_SYS_POP_INCORRECT_PASSWORD,
	PH_SYS_POP_ALREDY_EXISTS,
	PH_SYS_POP_SELECTED,
	CT_SYS_POP_SUCCESS,
	CT_SYS_POP_ERROR,
	CT_SYS_POP_MOVED,
	CT_SYS_POP_ADDED,
	CT_SYS_POP_NOT_ENOUGH_MEMORY,
	CT_SYS_POP_NOTHING_SELECTED,
	CT_SYS_POP_FILE_NOT_FOUND,
	CT_SYS_POP_ADDRESS,
	PH_SYS_OPT_ADD_TO_CONTACTS,
	PH_SYS_BODY_SELECTED_DATA_NOT_FOUND,
	PH_SYS_POP_DELETE_Q,
	CT_SYS_SK_SET,
	CT_SYS_SK_RESET,
	CT_SYS_OPT_MY_FILES,
	CT_SYS_DAYS_AGO,
	CT_SYS_BODY_SETTINGS,
	CT_SYS_BODY_BACK,
	CT_SYS_BODY_MORE,
	CT_SYS_BODY_NO_CONTENTS,
	CT_SYS_BODY_PD_SELECTED,
	CT_PB_BODY_ABCDEFGHIJKLMNOPQRSTUVWXYZ_M_INDEXLIST,
	CT_SYS_POP_MAXIMUM_NUMBER_OF_CHARACTERS_REACHED,
	CT_SYS_SK_SELECT,
	CT_SYS_OPT_S_MEMO,
	CT_SYS_BODY_CONTINUE,
	CT_SYS_POP_CANCELLED,
};

#ifdef __cplusplus
}
#endif

#endif // __CT_STRING_H__

