/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _CT_TYPE_H_
#define _CT_TYPE_H_

#include <app_control.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define CT_EDITFIELD_MAX_CHAR 1000
#define CT_MAIN_NAME_EDITFIELD_MAX_CHAR 5005
#define CT_SIMNAME_EDITFIELD_MAX_CHAR 14
#define CT_SIMNUMBER_EDITFIELD_MAX_CHAR 40


#define CT_TEXT_MAX_LEN 1024
#define CT_HELP_TEXT_MAX_LEN 4096
#define CT_TEXT_SHORT_LEN 256
#define CT_EDITFIELD_MAX_BUFFER 1000*3
#define CT_SEARCHBAR_MAX_CHAR 100

#define CT_KEY_APPSVC_TIMER "appsvc_timer"
#define CT_KEY_SERVICE "app_control_h"
#define CT_KEY_LIST_MFC_PERSON_IDS "db/private/org.tizen.contacts/list_mfc_person_ids"
#define CT_KEY_LIST_DISPLAY_ADDRESSBOOK "db/private/org.tizen.contacts/list_display_addressbook"
#define CT_KEY_INPUT_BASE_ADDRESSBOOK "db/private/org.tizen.contacts/input_base_addressbook"
#define CT_KEY_SETTING_CONTACT_SHARING_SETTINGS "db/private/org.tizen.contacts/contact_sharing_settings"
#define CT_KEY_CONTACT_CREATE_FIRST "db/private/org.tizen.contacts/contact_create_first"
#define CT_KEY_MYPROFILE_CREATE_FIRST "db/private/org.tizen.contacts/myprofile_create_first"
#define CT_KEY_DISPLAY_WITH_NUMBER "db/private/org.tizen.contacts/display_contacts_with_number"
#define CT_KEY_LAST_SELECTED_TAB_INDEX_FOR_PHONE "db/private/org.tizen.contacts/last_selected_tab_index_for_phone"
#define CT_KEY_SHOW_LINK_CONTACTS_TOAST_POPUP "db/private/org.tizen.contacts/show_link_contacts_toast_popup"
#define CT_KEY_NUM_OF_LINKABLE_CONTACTS "db/private/org.tizen.contacts/num_of_linkable_contacts"
#define CT_SIM_ADDRESSBOOK_NAME "http://tizen.samsung.com/addressbook/sim"
#define CT_EAS_ADDRESSBOOK_NAME "Microsoft Exchange ActiveSync"
#define CT_EAS_ADDRESSBOOK_DISPLAY_NAME "Exchange ActiveSync"
#define CT_FACEBOOK_ADDRESSBOOK_NAME "Sync apps with Facebook"

#define CT_KEY_SIM_CARD_NUMBER "sim_card_number"

//////////////////////// List Data..We need to hide these enum into list data class////////////////////////
typedef struct {
	int addressbookId;
	int accountId;
	char *iconPath;
	char *addressbookName;
	char *displayName;
	bool display;
	int mode;
} CtAddressbookInfo;

typedef enum {
	LIST_ADDRESSBOOK_MODE_NONE,
	LIST_ADDRESSBOOK_MODE_ALL,
	LIST_ADDRESSBOOK_MODE_VISIBLE,
	LIST_ADDRESSBOOK_MODE_INVISIBLE,
}CtListAddressbookMode;


//////////////////////////////////////////////////////////////////////////////////////////////////

typedef enum {
	LIST_DATA_DEFAULT,//Add enumtype refer to op
	LIST_DATA_ALL_PERSON,
	LIST_DATA_ALL_FAVORITES,
	LIST_DATA_PERSON_EXCEPT_SIM,//without sim
	//Carrier matching
	LIST_DATA_PERSON_WITH_NUMBER,
	LIST_DATA_PERSON_EXCEPT_FACEBOOK,

}CtListDataType;

typedef enum {
	LIST_CTRL_DEFAULT, //go to detail
	LIST_CTRL_DELETE_CONTACTS,
	LIST_CTRL_SHARE_CONTACTS,
	LIST_CTRL_ADD_FAVORITES,
	LIST_CTRL_REMOVE_FAVORITES,
	LIST_CTRL_REORDER_FAVORITES,
	LIST_CTRL_LINK_CONTACT,

	// Brazil carrier matching
	LIST_CTRL_CARRIER_MATCH_SYNC_CONTACTS,

	//for app control
	LIST_CTRL_REQUEST_UPDATE_CONTACT, //for choose

	LIST_CTRL_REQUEST_SELECT_FOR_PERSON, //for pick single
	LIST_CTRL_REQUEST_SELECT_FOR_NUMBER, //
	LIST_CTRL_REQUEST_SELECT_FOR_EMAIL, //
	LIST_CTRL_REQUEST_SELECT_FOR_NUMBER_OR_EMAIL, //
	LIST_CTRL_REQUEST_SELECT_FOR_URL, //
	LIST_CTRL_REQUEST_SELECT_FOR_VCARD, //

	LIST_CTRL_REQUEST_CHECK_FOR_PERSON, //for pick multiple
	LIST_CTRL_REQUEST_CHECK_FOR_NUMBER, //
	LIST_CTRL_REQUEST_CHECK_FOR_EMAIL, //
	LIST_CTRL_REQUEST_CHECK_FOR_VCARD, //
	LIST_CTRL_REQUEST_CHECK_FOR_NUMBER_OR_EMAIL, //
	LIST_CTRL_REQUEST_CHECK_FOR_LINK_CONTACTS,

	LIST_CTRL_SHARE_CONTACT_INFO,
}CtListCtrlType;

typedef enum {
	LIST_VIEW_DEFAULT,
	LIST_VIEW_CHECK,
	LIST_VIEW_REORDER,
}CtListViewType;

typedef enum {
	CHECK_NONE = -1,
	CHECK_OFF,
	CHECK_ON,
}CtCheckState;

typedef enum {
	RESULT_FAILED = APP_CONTROL_RESULT_FAILED,
	RESULT_SUCCESS = APP_CONTROL_RESULT_SUCCEEDED,
	RESULT_CANCELLED = APP_CONTROL_RESULT_CANCELED
} CtResultType;

enum {
	INPUT_ADDED_ITEM_INDEX_NONE = 0,
	INPUT_ADDED_ITEM_INDEX_PHONETIC_INFO,
	INPUT_ADDED_ITEM_INDEX_NICKNAME,
	INPUT_ADDED_ITEM_INDEX_ORGANIZATION,
	INPUT_ADDED_ITEM_INDEX_NUMBER,
	INPUT_ADDED_ITEM_INDEX_EMAIL,
	INPUT_ADDED_ITEM_INDEX_NOTE,
	INPUT_ADDED_ITEM_INDEX_URL,
	INPUT_ADDED_ITEM_INDEX_ADDRESS,
	INPUT_ADDED_ITEM_INDEX_IM,
	INPUT_ADDED_ITEM_INDEX_RELATIONSHIP,
};

typedef enum {
	CTTEXT_TYPE_CUSTOM = 1,
	CTTEXT_TYPE_MOBILE,
	CTTEXT_TYPE_WORK,
	CTTEXT_TYPE_HOME,
	CTTEXT_TYPE_MAIN,
	CTTEXT_TYPE_FAX_WORK,
	CTTEXT_TYPE_FAX_HOME,
	CTTEXT_TYPE_PAGER,
	CTTEXT_TYPE_OTHER,
}CtInputNumberDataType;

typedef enum {
	CTTEXT_EMAIL_TYPE_CUSTOM,
	CTTEXT_EMAIL_TYPE_HOME,
	CTTEXT_EMAIL_TYPE_WORK,
	CTTEXT_EMAIL_TYPE_OTHER,
}CtInputEmailDataType;

typedef enum {
	CTTEXT_MESSENGER_TYPE_CUSTOM,
	CTTEXT_MESSENGER_TYPE_AIM,
	CTTEXT_MESSENGER_TYPE_YAHOO,
	CTTEXT_MESSENGER_TYPE_SKYPE,
	CTTEXT_MESSENGER_TYPE_QQ,
	CTTEXT_MESSENGER_TYPE_HANGOUTS,
	CTTEXT_MESSENGER_TYPE_ICQ,
	CTTEXT_MESSENGER_TYPE_JABBER,
}CtInputMessengerDataType;

typedef enum {
	LIST_QUERY_DEFAULT,
	LIST_QUERY_FAVORITE,
	LIST_QUERY_MFC,
}CtListQueryType;

#ifdef __cplusplus
}
#endif

#endif
