/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _CT_INPUT_VIEW_H_
#define _CT_INPUT_VIEW_H_

#include <Elementary.h>

#include <contacts.h>

#include "WNaviframe.h"
#include "WPopup.h"
#include "WView.h"

#include "ContactsCommon.h"

#include "IContactsTabPageView.h"
#include "ICtChangeListener.h"
#include "CtEditTabView.h"
#include "CtType.h"
#include "CtView.h"
#include "CtThread.h"

class CtData;
class CtInputViewControl;
class CtEditTabView;

class WAPP_ASSIST_EXPORT CtInputView: public CtView, public IContactsTabPageView, public ICtChangeListener
{
public:
	CtInputView(contacts_record_h contactRecord);
	CtInputView(contacts_record_h contactRecord, CtEditTabView* tabView, bool isSubView);
	CtInputView(int personId, const char* number, const char* email, const char* uri);
	CtInputView(const char* name, const char* number, const char* email, const char* uri);

public:
	bool getIsCreate() { return __isCreate; }
	bool getIsDestroy() { return __destroy; }
	Evas_Object* getNaviframeEvasObj();
	Evas_Object* getWindowEvasObj();
	Evas_Object* getConformantEvasObject();
	void setOnResult(const std::function<void(int, CtResultType)>& handlerFunc);

	void createEditSubViewContent();
	void createInputViewControl(Evas_Object *base);
	void updateSaveBtn(Eina_Bool empty);
	void updateNumberSaveBtn(Eina_Bool isEmpty);
	void saveContactForEditTabSubView();
	void getChangedForEditTabSubView(bool* changed);
	void getEmptyForEditTabSubView(bool* empty);
	virtual void onTabSelect(Elm_Object_Item *naviItem);
	virtual void onTabUnselect(Elm_Object_Item *naviItem);
	virtual bool onTabPop();
	virtual void onTabScrollStart() {return;}
	virtual void onTabScrollStop() {return;}
	void resetTheme();

protected:
	virtual Evas_Object* onCreate(Evas_Object* parent, void* viewParam);
	virtual void onDestroy();
	virtual void onPushed(Elm_Object_Item* naviItem);
	virtual void onEvent(int eventType);
	virtual bool onPop();

protected:
	virtual void onLanguageChanged();

private:

	virtual void onDbChanged(const char* viewUri);
	virtual void onCompleteGetContacts(int offset);
	virtual void onConditionChagned();

	void __initialize();

	void __createToastPopup(const char* msg);
	static void __saveCb(void *data, Evas_Object *obj, void *event_info);
	static void __cancelCb(void *data, Evas_Object *obj, void *event_info);
	int __getPersonIdByContactId(int contactId);

	static Eina_Bool __createAddressbookPopupIdlerCb(void *data);

private:
	CtInputView();
	virtual ~CtInputView();

private:

	WUiObjectPtr __inputControl;
	CtEditTabView *__tabView;
	std::shared_ptr<CtData> __inputData;

	int __savedAddressbookId;
	bool __isCreate;
	bool __saved;
	bool __discard;
	bool __changed;
	bool __destroy;

	Evas_Object* __saveBtn;
	Evas_Object* __baseLayout;

	Ecore_Idler *__popupIdler;
	Ecore_Event_Handler* __windowVisibilityChangeHandler;
	std::function<void(int, CtResultType)> __resultHandler;
};

#endif /* _CT_INPUT_VIEW_H_ */
