/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

//#include <ui_extension.h>

#include "WPageSlideView.h"

#include "ContactsDebug.h"
//#include "ContactsAppControlLauncher.h"

#include "CtCommon.h"
#include "CtString.h"
#include "CtEditTabView.h"
#include "CtInputView.h"

#define CT_TAB_CREATE_SUB_VIEW_TIMER 1.0

CtEditTabView::CtEditTabView(Eina_List* contactList)
{
	WENTER();
	__idler = NULL;
	__timer = NULL;
	__tabbar = NULL;
	__tabCount = eina_list_count(contactList);
	__discard = false;
    __selectedView = 0;

	int i = 0;
	Eina_List *l = NULL;
	void* data = NULL;
	char buf[CT_TEXT_MAX_LEN] = {0};
	WDEBUG("__tabCount %d", __tabCount);
	if (__tabCount > EDIT_TAB_VIEW_MAX) {
		__tabCount = EDIT_TAB_VIEW_MAX;
	}

	for( i=0; i<__tabCount; i++) {
		__tabItem[i] = NULL;
		__subViewLoaded[i] = false;
		__subView[i] = NULL;
	}
	i = 0;
	EINA_LIST_FOREACH(contactList, l, data) {
		contacts_record_h record = (contacts_record_h)data;
		if (NULL == record) break;
		contacts_record_h recordCloned = NULL;
		int err = contacts_record_clone(record, &recordCloned);
		WPRDT_M(CONTACTS_ERROR_NONE != err, "contacts_record_clone() Failed(%d)", err);
		__subView[i] = new CtInputView(recordCloned, this, true);
		__getTabViewTitle(recordCloned, buf, sizeof(buf));
		__subView[i]->setName(buf);
		i++;
		if(i == EDIT_TAB_VIEW_MAX)
			break;
	}

	WLEAVE();
}

CtEditTabView::~CtEditTabView()
{
	WHIT();
}


Evas_Object* CtEditTabView::onCreate(Evas_Object* parent, void* viewParam)
{
	WENTER();

	Evas_Object* layout = elm_layout_add(parent);
	elm_layout_theme_set(layout, "layout", "application", "default");
	evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);

	setOnInitScroller( [](Evas_Object* scroller)
	{
		elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_OFF);
		//uxt_scroller_set_auto_scroll_enabled(scroller, EINA_FALSE);
	});

	Evas_Object* slideViewObj = WPageSlideView::onCreate(layout, NULL);

	elm_object_part_content_set(layout, "elm.swallow.content", slideViewObj);
	evas_object_smart_callback_add(getScroller(), "scroll,anim,stop", __pageScrollStopCb, this );
	evas_object_smart_callback_add(getScroller(), "scroll,drag,start", __pageScrollStartCb, this );

	evas_object_show(layout);

	WDEBUG("layout %p", layout);

	return layout;

}


void CtEditTabView::onDestroy()
{
	if(__idler)
		ecore_idler_del(__idler);
	__idler = NULL;

	if(__timer)
		ecore_timer_del(__timer);
	__timer = NULL;
}

Elm_Object_Item* CtEditTabView::onPushToNaviframe(Evas_Object* naviFrame)
{
	WDEBUG("page set");

	//create tabbar
	Evas_Object* tabbar = elm_toolbar_add(naviFrame);
	elm_toolbar_shrink_mode_set(tabbar, ELM_TOOLBAR_SHRINK_EXPAND);
	elm_toolbar_select_mode_set(tabbar, ELM_OBJECT_SELECT_MODE_ALWAYS);
	elm_toolbar_transverse_expanded_set(tabbar, EINA_TRUE);

	Elm_Object_Item* naviItem = NULL;
	elm_object_style_set(tabbar, "tabbar_with_title");
	naviItem = elm_naviframe_item_push(naviFrame, V_("IDS_PB_OPT_EDIT"), NULL, NULL, getEvasObj(), "tabbar");

	elm_object_item_part_content_set(naviItem, "tabbar", tabbar);
	__tabbar = tabbar;
	for( int i = 0; i<__tabCount; i++) {
		const char *tabName = NULL;
		tabName = __subView[i]->getName();

		WDEBUG("tabName %s %d", tabName, i);
		__tabItem[i] = elm_toolbar_item_append(tabbar, NULL, tabName, __onTabChangedCb, this);
		elm_object_item_domain_text_translatable_set(__tabItem[i], PACKAGE, EINA_TRUE);
	}

	WLEAVE();

	return naviItem;

}

void CtEditTabView::onPushed(Elm_Object_Item* naviItem)
{
	WENTER();

	for( int i = 0; i<__tabCount; i++) {
		if(__subView[i])
			appendPage();
	}

	//set first show page
	if(__subView[__selectedView]) {
		setPageContent(__selectedView, __subView[__selectedView]);
		__subView[__selectedView]->createEditSubViewContent();
		__subViewLoaded[__selectedView] = true;
		//elm_toolbar_item_selected_set(__tabItem[__selectedView], EINA_TRUE);

	//	showPage(__selectedView, false);
		//__timer = ecore_timer_add(CT_TAB_CREATE_SUB_VIEW_TIMER, __createRestViewTimerCb, this);
	}

	__saveBtn = elm_button_add(getNaviframe()->getEvasObj());
	elm_object_style_set(__saveBtn, "naviframe/title_done");
	elm_object_text_set(__saveBtn, V_("IDS_TPLATFORM_ACBUTTON_DONE_ABB"));
	evas_object_smart_callback_add(__saveBtn, "clicked", __saveCb, this);
	elm_object_item_part_content_set(naviItem, "title_right_btn", __saveBtn);

	Evas_Object* cancelBtn = elm_button_add(getNaviframe()->getEvasObj());
	elm_object_style_set(cancelBtn, "naviframe/title_cancel");
	elm_object_text_set(cancelBtn, V_("IDS_TPLATFORM_ACBUTTON_CANCEL_ABB"));
	evas_object_smart_callback_add(cancelBtn, "clicked", __cancelCb, this);
	elm_object_item_part_content_set(naviItem, "title_left_btn", cancelBtn);
	WLEAVE();
}

bool CtEditTabView::onPop( )
{
	WHIT();
	bool changed = false;

	for( int i = 0; i<__tabCount; i++) {
		if (__subView[i]) {
			__subView[i]->onTabPop();
			bool changedSub = false;
			__subView[i]->getChangedForEditTabSubView(&changedSub);
			changed = changed | changedSub;
		}
	}
	if (__discard) {
		return true;
	}

	if (changed) {
		WDEBUG("changed");

		WPopup *popup = new WPopup();
		popup->setTextTranslatable(PACKAGE);
		popup->setTitle("IDS_PB_HEADER_DISCARD_CHANGES_ABB");
		popup->setContent("IDS_PB_POP_ALL_CHANGES_WILL_BE_DISCARDED");

		popup->addButton("IDS_PB_BUTTON_CANCEL", [](bool* destroyPopup){
			WDEBUG("Cancel");
			*destroyPopup = true;
		});
		popup->addButton(V_("IDS_PB_BUTTON_DISCARD_ABB"), [this](bool* destroyPopup){
			WDEBUG("Discard");
			__discard = true;
			popOut();
			*destroyPopup = false;
		});

		attachPopup(popup);
		return false;
	}

	return true;
}

void CtEditTabView::updateSaveBtn(Eina_Bool empty)
{
	if (!empty) {
		bool emptyAll = false;
		for( int i = 0; i<__tabCount; i++) {
			if (__subView[i]) {
				bool emptySub = false;
				__subView[i]->getEmptyForEditTabSubView(&emptySub);
				emptyAll = emptyAll | emptySub;
				if (emptyAll) {
					break;
				}
			}
		}
		empty = emptyAll;
	}
	elm_object_disabled_set(__saveBtn, empty);
	WPRET_M(true == empty, "all fields are empty, can't save");
}

void CtEditTabView::__saveCb(void *data, Evas_Object *obj, void *event_info)
{
	WHIT();
	CtEditTabView* view = (CtEditTabView*)data;

	for( int i = 0; i<view->__tabCount; i++) {
		if (view->__subView[i]) {
			bool changedSub = false;
			view->__subView[i]->getChangedForEditTabSubView(&changedSub);
			if (changedSub) {
				view->__subView[i]->saveContactForEditTabSubView();
			}
		}
	}
	view->destroyPopup();
	elm_object_item_del(view->getNaviItem());

	//view->popOut();
}

void CtEditTabView::__cancelCb(void *data, Evas_Object *obj, void *event_info)
{
	CtEditTabView* view = (CtEditTabView*)data;
	view->popOut();
}

void CtEditTabView::__onTabChangedCb(void* data, Evas_Object* obj, void* event_info)
{
	WHIT();
	CtEditTabView* view = (CtEditTabView*)data;
	Elm_Object_Item *selectedItem = elm_toolbar_selected_item_get(obj);

	view->__tabChanged(selectedItem);

}

void CtEditTabView::__tabChanged(Elm_Object_Item *selectedItem)
{
	WHIT();

	if(getPageCount() == 0) {
		WERROR("page item not added");
		return;
	}

	if(selectedItem != __tabItem[__selectedView]) {
		int i = 0;
		WDEBUG("%d", __selectedView);

		if(selectedItem) {

			if(__subView[__selectedView] && __subViewLoaded[__selectedView])
				__subView[__selectedView]->onTabUnselect(getNaviItem());

			for (i=0;i<__tabCount;i++) {
				if (__tabItem[i]) {
					if (selectedItem == __tabItem[i]) {
						if(__subView[i]) {
							setPageContent(i, __subView[i]);
							__subViewLoaded[i] = true;
							__subView[i]->onTabSelect(getNaviItem());
						}
						break;
					}
				}
			}

			__selectedView = i;
		}
		else {//first selected(launched)
			WDEBUG("first launched");
			if(__subView[__selectedView] && __subViewLoaded[__selectedView]) {
				__subView[__selectedView]->onTabSelect(getNaviItem());
			}
		}
	}

	showPage(__selectedView, false);

}

void CtEditTabView::__pageScrollStopCb(void* data, Evas_Object* obj, void* event_info)
{
	CtEditTabView* view = (CtEditTabView*)data;
	for(int i = 0; i < view->__tabCount; i++)	{
		if(view->__subView[i] && view->__subViewLoaded[i])
			view->__subView[i]->onTabScrollStop();
	}

	int hpage = 0;
	elm_scroller_current_page_get(obj, &hpage, NULL);
	WDEBUG("pagen %d", hpage);

	elm_toolbar_item_selected_set(view->__tabItem[hpage], EINA_TRUE);
	return;
}

void CtEditTabView::__pageScrollStartCb(void* data, Evas_Object* obj, void* event_info)
{
	WHIT();
	CtEditTabView* view = (CtEditTabView*)data;
	for( int i = 0; i<view->__tabCount; i++) {
		WDEBUG("subView %d", i);
		if(view->__subView[i] && view->__subViewLoaded[i] == false) {
			view->setPageContent(i, view->__subView[i]);
			view->__subViewLoaded[i] = true;
		}
		view->__subView[i]->onTabScrollStart();
    }

	return;
}

bool CtEditTabView::__checkViewExist(Elm_Object_Item *tabItem)
{
	bool result = false;
	int i = 0;
	for(i=0;i<__tabCount;i++) {
		if(__tabItem[i] == tabItem) {
			if(__subView[i]) {
				result = true;
				break;
			}
		}
	}

	return result;
}


void CtEditTabView::__getTabViewTitle(contacts_record_h recordContact, char *buf, int bufLen)
{
	int addressbookId = -1;
	int err = CONTACTS_ERROR_NONE;
	char *addressbookName = NULL;
	contacts_record_h recordAddressbook = NULL;

	buf[0] = '\0';

	err = contacts_record_get_int(recordContact, _contacts_contact.address_book_id, &addressbookId);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);

	err = contacts_db_get_record(_contacts_address_book._uri, addressbookId, &recordAddressbook);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

	ctCommonGetProviderNameByAddressbookId(recordAddressbook, addressbookId, &addressbookName, NULL);
	WDEBUG("addressbookName %s ", addressbookName);

	err = contacts_record_destroy(recordAddressbook, true);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_destroy() Failed(%d)", err);

	snprintf(buf, bufLen, "%s", addressbookName);

	free(addressbookName);
}

Eina_Bool CtEditTabView::__createRestViewTimerCb(void* data)
{
	WHIT();
	CtEditTabView* view = (CtEditTabView*)data;
	view->__timer = NULL;

	view->__idler = ecore_idler_add([](void* data)->Eina_Bool {
		CtEditTabView* view = (CtEditTabView*)data;

		for( int i = 0; i<view->__tabCount; i++) {
			WDEBUG("subView %d", i);
			if(view->__subView[i] && view->__subViewLoaded[i] == false) {
				view->setPageContent(i, view->__subView[i]);
				view->__subView[i]->createEditSubViewContent();
				view->__subViewLoaded[i] = true;
			}
	    }
	    view->__idler = NULL;

		return ECORE_CALLBACK_CANCEL;
	}, view );

	return ECORE_CALLBACK_CANCEL;
}

