/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _CT_SETTING_CONTACT_IMPORTER_H_
#define _CT_SETTING_CONTACT_IMPORTER_H_

#include <string>
#include <vector>

#include "WDefine.h"

class CtProgressBarPopup;

class CtImporter {
public:
	CtImporter(int abId, std::vector<std::string> vcards, int contactsCount);
	~CtImporter();

	CtProgressBarPopup *getProgressPopup();
	WPopup *getCancelPopup();
	void run();

private:
	void __createProgressPopup();
	void __createCancelPopup();
	void __cancelImporting(bool* destroyPopup);
	void __stopImporting(bool* destroyPopup);
	void __continueImporting(bool* destroyPopup);
	static void __importFromVcardsCb(void *data, Ecore_Thread *thread);
	static void __importFromVcardsEndCb(void *data, Ecore_Thread *thread);
	static void __importFromVcardsCancelCb(void *data, Ecore_Thread *thread);
	static bool __vcardParseCb(contacts_record_h record, void *user_data);

private:
	WDISABLE_COPY_AND_ASSIGN(CtImporter);

private:
	int __addressBookId;
	bool __canceled;
	bool __paused;
	std::vector<int> __importedContacts;
	CtProgressBarPopup *__progressPopup;
	WUiObjectPtr __popupMonitor;
	Ecore_Thread *__importThread;
	std::vector<std::string> __vcards;
	WPopup *__cancelPopup;
	int __contactsCount;
};

#endif //_CT_SETTING_CONTACT_IMPORTER_H_
