/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <vector>
#include <thread>
#include <Elementary.h>
#include <app.h>
#include <contacts.h>

#include "WView.h"
#include "WNaviframe.h"
#include "WPopup.h"

#include "ContactsDebug.h"
#include "CtType.h"
#include "CtListDataPerson.h"
#include "CtCommon.h"
#include "CtListView.h"
#include "CtListModelDb.h"
#include "CtProgressBarPopup.h"
#include "CtListLinkContactController.h"

CtListLinkContactController::CtListLinkContactController(CtListView *listView, CtListModelDb* listData)
: __listView(listView)
, __listModel(listData)
, __checkedItemCount(0)
, __basePersonId(0)
{

}

CtListLinkContactController::~CtListLinkContactController()
{
	WHIT();
}

void CtListLinkContactController::onSelectItem(Elm_Object_Item *selectedItem, CtListView *view, CtCheckState checkState)
{
	Elm_Object_Item *item = selectedItem;
	elm_genlist_item_selected_set(item, EINA_FALSE);

	Elm_Genlist_Item_Type item_type = elm_genlist_item_type_get(item);
	if(item_type == ELM_GENLIST_ITEM_GROUP) {
		return;
	}

	Evas_Object* layout = elm_object_item_part_content_get(item,"elm.icon.2");
	Evas_Object* check = elm_layout_content_get(layout, "elm.swallow.content");

	CtListView::CtListDataItem* itemData = static_cast<CtListView::CtListDataItem*>(elm_object_item_data_get(item));
	WPRET_M(itemData == NULL, "itemData is null");

	if(checkState == CHECK_ON) {
		itemData->personData->setAsChecked(true);
		elm_check_state_set(check, true);
		__checkedItemCount++;
	}
	else if(checkState == CHECK_OFF) {
		itemData->personData->setAsChecked(false);
		elm_check_state_set(check, false);
		__checkedItemCount--;
	}

	Elm_Object_Item *naviItem = elm_naviframe_top_item_get(__listView->getViewNaviframe());
	Evas_Object *btn = elm_object_item_part_content_get(naviItem, "title_right_btn");
	if(__checkedItemCount > 0)
		elm_object_disabled_set(btn, EINA_FALSE);
	else
		elm_object_disabled_set(btn, EINA_TRUE);

	char buf[CT_TEXT_SHORT_LEN] = {0,};
	snprintf(buf, sizeof(buf), ctGetContactsString("IDS_PB_HEADER_PD_SELECTED_ABB3"), __checkedItemCount);
	elm_object_item_part_text_set(naviItem, "elm.text.title", buf );

	WHIT();
}

void CtListLinkContactController::onSelectCheckAll(Elm_Object_Item *selectedItem, CtListView *view, CtCheckState checkState)
{
}

void CtListLinkContactController::onSelectDone(void *data, Evas_Object *obj, void *event_info)
{
	WHIT();

	int err = CONTACTS_ERROR_NONE;
	std::vector<std::shared_ptr<CtListDataPerson>>* checkedList = __listModel->getCheckedDataList();
	for (auto iter = checkedList->begin(); iter != checkedList->end(); iter++) {
		WDEBUG("Link %d to %d", (*iter)->getPersonId(), __basePersonId);
		err = contacts_person_link_person(__basePersonId, (*iter)->getPersonId());
		WPWARN(err != CONTACTS_ERROR_NONE, "contacts_person_link_person() fail(%d)",err);
	}

	delete checkedList;

	__listView->popOut();
}

void CtListLinkContactController::onSelectCancel(void *data, Evas_Object *obj, void *event_info)
{
	WHIT();
	__listView->popOut();
}

void CtListLinkContactController::onSelectBack(void *data, Evas_Object *obj, void *event_info)
{
	WHIT();
}

void CtListLinkContactController::onSwipeLeftItem(void *data, Evas_Object *obj, void *event_info)
{
	// empty
}

void CtListLinkContactController::onSwipeRightItem(void *data, Evas_Object *obj, void *event_info)
{
	// empty
}

void CtListLinkContactController::moreManageFavoritesItemOperation()
{
	// empty
}
