/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef __CT_LIST_MODEL_H_
#define __CT_LIST_MODEL_H_

#include <vector>
#include <memory>
#include "CtListDataPerson.h"
#include "ICtChangeListener.h"

typedef enum {
	GET_AS_SYNC,
	GET_AS_ASYNC,
	GET_AS_SYNC_ASYNC,
}CtGetContactsType;


class CtListModel
{
public:
	virtual ~CtListModel(){};

public:
	virtual int getContactsCount(CtListQueryType queryType) = 0; //get contacts count from db
	virtual int getContacts(CtListQueryType queryType, CtGetContactsType syncType) = 0;
	virtual int cancelGetContacts() = 0;
	virtual std::shared_ptr<CtListDataPerson> getAtPersonData(int index, CtListQueryType queryType) = 0;
	virtual int getPersonListCount(CtListQueryType queryType) = 0; //get contacts count from list
	virtual bool getMadePersonListStatus() {return true;}//get status of whether personlist has been updated from DB
	virtual void attachListener(ICtChangeListener* listener) = 0;

	virtual void setSelectedPersonId(int selectedPersonId) = 0;
	virtual const char* getSearchString(void) = 0;
	virtual bool setSearchString(char* search) = 0;
	virtual int getCheckedDataCount()= 0;
	virtual void getNumberSearchString(const std::shared_ptr<CtListDataPerson>& dataPerson, std::string &numberStr) = 0;

	virtual void initialOriginalCheckedIdList() = 0;
	virtual void clearOriginalCheckedIdList() = 0;
	virtual bool checkIsSelectAllOnSearching() = 0;
	virtual void initialOriginalPersonDataListSize() = 0;
	virtual void clearOriginalPersonDataListSize() = 0;
	virtual int getOriginalPersonDataListSize() = 0;
	virtual int getLimitCount() = 0;
	virtual int getCurrentLinkedCount() = 0;
};

#endif
