/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include <thread>
#include <contacts.h>
#include <algorithm>

#include "ContactsDebug.h"
#include "CtType.h"
#include "ContactsCommon.h"
#include "CtListDataPerson.h"
#include "CtListModelLinkSuggest.h"

CtListModelLinkSuggest::CtListModelLinkSuggest(const int selectedPersonId)
: __listener(NULL)
{
	WENTER();

	__listener = NULL;
	__selectedName = NULL;
	__selectedPersonId = selectedPersonId;

	__nameQuery = NULL;
	__emailQuery = NULL;
	__numberQuery = NULL;

	__nameList = NULL;
	__emailList = NULL;
	__numberList = NULL;

	__setLinkContactData(__selectedPersonId);
	__makeQuery();

	__getQueryResult();
	__makePersonList();


}

CtListModelLinkSuggest::~CtListModelLinkSuggest()
{
	WENTER();
	int err = CONTACTS_ERROR_NONE;

	err = contacts_list_destroy(__nameList, true);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);
	err = contacts_list_destroy(__emailList, true);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);
	err = contacts_list_destroy(__numberList, true);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_destroy() Failed(%d)", err);

	err = contacts_query_destroy(__nameQuery);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
	err = contacts_query_destroy(__emailQuery);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);
	err = contacts_query_destroy(__numberQuery);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_destroy() Failed(%d)", err);


	for(auto iter = __vcSelectedNumber.begin(); iter != __vcSelectedNumber.end(); iter++) {
		free(*iter);
	}

	for(auto iter = __vcSelectedEmail.begin(); iter != __vcSelectedEmail.end(); iter++) {
		free(*iter);
	}
	__vcSelectedEmail.clear();
	__vcSelectedNumber.clear();
	__personDataList.clear();

}

int CtListModelLinkSuggest::getContactsCount(CtListQueryType queryType)
{

	return getPersonListCount(LIST_QUERY_DEFAULT);
}

int CtListModelLinkSuggest::getContacts(CtListQueryType queryType, CtGetContactsType syncType)
{
	if(queryType == LIST_QUERY_DEFAULT)
		return __personDataList.size();
	else
		WERROR("wrong queryType %d", queryType);

	return 0;
}

std::shared_ptr<CtListDataPerson> CtListModelLinkSuggest::getAtPersonData(int index, CtListQueryType queryType)
{
	if(queryType == LIST_QUERY_DEFAULT) {
		if(index < (int)__personDataList.size())
			return __personDataList[index].second;
		else
			return NULL;
	}
	else {
		return NULL;
	}
}

int CtListModelLinkSuggest::getPersonListCount(CtListQueryType queryType)
{
	WENTER();
/*
	int err = CONTACTS_ERROR_NONE;


	int nameCount =0;
	int numberCount =0;
	int emailCount =0;


	err = contacts_db_get_count_with_query(__nameQuery, &nameCount);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_count_with_query() Failed(%d)", err);
	err = contacts_db_get_count_with_query(__numberQuery, &numberCount);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_count_with_query() Failed(%d)", err);
	err = contacts_db_get_count_with_query(__emailQuery, &emailCount);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_count_with_query() Failed(%d)", err);
*/
	//todo
	return __personDataList.size();
}

void CtListModelLinkSuggest::attachListener(ICtChangeListener* listener)
{
	WHIT();
	__listener = listener;
}

void CtListModelLinkSuggest::__setLinkContactData(int personId)
{
	WENTER();

	int err = CONTACTS_ERROR_NONE;
	contacts_record_h contactRecord = NULL;
	contacts_record_h selectedRecord = NULL;
	contacts_record_h nameRecord = NULL;
	contacts_record_h emailRecord = NULL;
	contacts_record_h numberRecord = NULL;
	char *value = NULL;
	int contactId =0;
	int count =0;


	err = contacts_db_get_record(_contacts_person._uri,personId, &selectedRecord);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);
	err = contacts_record_get_int(selectedRecord, _contacts_person.display_contact_id, &contactId);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
	err = contacts_db_get_record(_contacts_contact._uri,contactId, &contactRecord);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_record() Failed(%d)", err);

	err = contacts_record_get_child_record_at_p(contactRecord,_contacts_contact.name,0,&nameRecord);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);
	err = contacts_record_get_str(nameRecord,_contacts_name.first, &__selectedName);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);


	err = contacts_record_get_child_record_count(contactRecord,_contacts_contact.number,&count);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);
	for(int i = 0; i < (int)count ; i++){
		err = contacts_record_get_child_record_at_p(contactRecord,_contacts_contact.number, i, &numberRecord);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);
		err = contacts_record_get_str(numberRecord,_contacts_number.number, &value);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);
		__vcSelectedNumber.push_back(value);
	}

	err = contacts_record_get_child_record_count(contactRecord,_contacts_contact.email,&count);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);
	for(int i = 0; i < (int)count ; i++){
		err = contacts_record_get_child_record_at_p(contactRecord,_contacts_contact.email, i, &emailRecord);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_child_record_at_p() Failed(%d)", err);
		err = contacts_record_get_str(emailRecord,_contacts_email.email, &value);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_str() Failed(%d)", err);
		__vcSelectedEmail.push_back(value);
	}

}
int CtListModelLinkSuggest::__makeQuery()
{

	contacts_filter_h filter = NULL;
	int err = CONTACTS_ERROR_NONE;

	unsigned int personProjection[] = {
		_contacts_person.id,
		_contacts_person.addressbook_ids,
		_contacts_person.display_name,
		_contacts_person.has_phonenumber,
		_contacts_person.image_thumbnail_path,
		_contacts_person.display_name_index,
		_contacts_person.is_favorite,
	};

	unsigned int numberProjection[] = {
		_contacts_person_number.person_id,
		_contacts_person_number.display_name,
		_contacts_person_number.display_contact_id,
		_contacts_person_number.has_phonenumber,
		_contacts_person_number.image_thumbnail_path,
		_contacts_person_number.is_favorite,
		_contacts_person_number.display_name_index,
	};

	unsigned int emailProjection[] = {
		_contacts_person_email.person_id,
		_contacts_person_email.display_name,
		_contacts_person_email.display_contact_id,
		_contacts_person_email.has_phonenumber,
		_contacts_person_email.image_thumbnail_path,
		_contacts_person_email.is_favorite,
		_contacts_person_email.display_name_index,
	};

	if(__selectedName != NULL){
		err = contacts_query_create(_contacts_person._uri, &__nameQuery);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		err = contacts_filter_create(_contacts_person._uri, &filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_create() Failed(%d)", err);
		err = contacts_filter_add_str(filter, _contacts_person.display_name, CONTACTS_MATCH_CONTAINS, __selectedName);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_add_str() Failed(%d)", err);
		err = contacts_query_set_filter(__nameQuery, filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_set_filter() Failed(%d)", err);
		err = contacts_query_set_projection(__nameQuery, personProjection, sizeof(personProjection)/sizeof(int));
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_set_projection() Failed(%d)", err);
		err = contacts_filter_destroy(filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);
	}

	if(__vcSelectedNumber.size() > 0){
		err = contacts_query_create(_contacts_person_number._uri, &__numberQuery);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		err = contacts_filter_create(_contacts_person_number._uri, &filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_create() Failed(%d)", err);
		for(auto iter = __vcSelectedNumber.begin(); iter != __vcSelectedNumber.end(); iter++) {

			if(iter != __vcSelectedNumber.begin()){
				err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_OR);
				WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_add_operator() Failed(%d)", err);
			}

			err = contacts_filter_add_str(filter, _contacts_person_number.number, CONTACTS_MATCH_EXACTLY, *iter);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_add_str() Failed(%d)", err);
		}
		err = contacts_query_set_filter(__numberQuery, filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "__list_create_filter_bool() Failed(%d)", err);
		err = contacts_query_set_projection(__numberQuery, numberProjection, sizeof(numberProjection)/sizeof(int));
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_set_projection() Failed(%d)", err);
		err = contacts_filter_destroy(filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);

	}

	if(__vcSelectedEmail.size() > 0){
		err = contacts_query_create(_contacts_person_email._uri, &__emailQuery);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_create() Failed(%d)", err);
		err = contacts_filter_create(_contacts_person_email._uri, &filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_create() Failed(%d)", err);
		for(auto iter = __vcSelectedEmail.begin(); iter != __vcSelectedEmail.end(); iter++) {

			if(iter != __vcSelectedEmail.begin()){
				err = contacts_filter_add_operator(filter, CONTACTS_FILTER_OPERATOR_OR);
				WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_add_operator() Failed(%d)", err);
			}

			err = contacts_filter_add_str(filter, _contacts_person_email.email, CONTACTS_MATCH_EXACTLY, *iter);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_add_str() Failed(%d)", err);
		}
		err = contacts_query_set_filter(__emailQuery, filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "__list_create_filter_bool() Failed(%d)", err);
		err = contacts_query_set_projection(__emailQuery, emailProjection, sizeof(emailProjection)/sizeof(int));
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_query_set_projection() Failed(%d)", err);
		err = contacts_filter_destroy(filter);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_filter_destroy() Failed(%d)", err);


	}
	return err;
}

int CtListModelLinkSuggest::__getQueryResult()
{
	WENTER();
	int err = CONTACTS_ERROR_NONE;


	int totalCount =0;
	int count =0;

	err = contacts_db_get_records_with_query(__nameQuery, 0, 0, &__nameList);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_records_with_query() Failed(%d)", err);

	err = contacts_db_get_records_with_query(__numberQuery, 0, 0, &__numberList);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_records_with_query() Failed(%d)", err);

	err = contacts_db_get_records_with_query(__emailQuery, 0, 0, &__emailList);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_db_get_records_with_query() Failed(%d)", err);

	err = contacts_list_get_count(__nameList, &count);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_get_count() Failed(%d)", err);
	totalCount += count;

	err = contacts_list_get_count(__numberList, &count);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_get_count() Failed(%d)", err);
	totalCount += count;

	err = contacts_list_get_count(__emailList, &count);
	WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_get_count() Failed(%d)", err);
	totalCount += count;

	if(totalCount > 0){
		__makePersonList();
	}
	return err;

}

int	CtListModelLinkSuggest::__makePersonList()
{
	contacts_record_h record = NULL;
	int err = CONTACTS_ERROR_NONE;

	while (CONTACTS_ERROR_NONE == contacts_list_get_current_record_p(__nameList, &record)) {
		if(record){
			std::shared_ptr<CtListDataPerson> dataPerson(std::make_shared<CtListDataPerson>(record));

			int personId = 0;
			err = contacts_record_get_int(record, _contacts_person.id, &personId);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
			if (__selectedPersonId == personId) {
				contacts_list_next(__nameList);
				continue;
			}
			if(__selectedPersonId == personId) {
				contacts_list_next(__nameList);
				continue;
			}
			std::vector<std::pair<int, std::shared_ptr<CtListDataPerson>>>::iterator existIter = std::find_if( __personDataList.begin(), __personDataList.end(),
					[personId](std::pair<int, std::shared_ptr<CtListDataPerson>> dataPerson)->bool {
					if(dataPerson.first == personId) {
						return true;
					}
					return false;
				}
			);
			if(existIter != __personDataList.end()){
				contacts_list_next(__nameList);
				continue;
			}
			__personDataList.push_back(std::make_pair(dataPerson->getPersonId(), dataPerson));
		}
		err = contacts_list_next(__nameList);
		WPWARN(CONTACTS_ERROR_NONE != err, "contacts_list_next() Failed(%d)", err);
	}

	while (CONTACTS_ERROR_NONE == contacts_list_get_current_record_p(__numberList, &record)) {
		if(record){
			std::shared_ptr<CtListDataPerson> dataPerson(std::make_shared<CtListDataPerson>(record));

			int personId = 0;
			err = contacts_record_get_int(record, _contacts_person_number.person_id, &personId);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
			if (__selectedPersonId == personId) {
				contacts_list_next(__numberList);
				continue;
			}
			if(__selectedPersonId == personId) {
				contacts_list_next(__numberList);
				continue;
			}

			std::vector<std::pair<int, std::shared_ptr<CtListDataPerson>>>::iterator existIter = std::find_if( __personDataList.begin(), __personDataList.end(),
					[personId](std::pair<int, std::shared_ptr<CtListDataPerson>> dataPerson)->bool {
					if(dataPerson.first == personId) {
						return true;
					}
					return false;
				}
			);
			if(existIter != __personDataList.end()){
				contacts_list_next(__numberList);
				continue;
			}
			__personDataList.push_back(std::make_pair(dataPerson->getPersonId(), dataPerson));
		}
		err = contacts_list_next(__numberList);
	}

	while (CONTACTS_ERROR_NONE == contacts_list_get_current_record_p(__emailList, &record)) {
		if(record){
			std::shared_ptr<CtListDataPerson> dataPerson(std::make_shared<CtListDataPerson>(record));

			int personId = 0;
			err = contacts_record_get_int(record, _contacts_person_email.person_id, &personId);
			WPWARN(CONTACTS_ERROR_NONE != err, "contacts_record_get_int() Failed(%d)", err);
			if (__selectedPersonId == personId) {
				contacts_list_next(__emailList);
				continue;
			}
			if(__selectedPersonId == personId) {
				contacts_list_next(__emailList);
				continue;
			}

			std::vector<std::pair<int, std::shared_ptr<CtListDataPerson>>>::iterator existIter = std::find_if( __personDataList.begin(), __personDataList.end(),
					[personId](std::pair<int, std::shared_ptr<CtListDataPerson>> dataPerson)->bool {
					if(dataPerson.first == personId) {
						return true;
					}
					return false;
				}
			);
			if(existIter != __personDataList.end()){
				contacts_list_next(__emailList);
				continue;
			}
			__personDataList.push_back(std::make_pair(dataPerson->getPersonId(), dataPerson));
		}
		err = contacts_list_next(__emailList);
	}


	return 0;
}

void CtListModelLinkSuggest::setSelectedPersonId(int selectedPersonId)
{
	// empty
}

const char* CtListModelLinkSuggest::getSearchString(void)
{
	return NULL;
}

bool CtListModelLinkSuggest::setSearchString(char* search)
{
	return false;
}

void CtListModelLinkSuggest::getNumberSearchString(const std::shared_ptr<CtListDataPerson>& dataPerson, std::string &numberStr)
{

}

int CtListModelLinkSuggest::getCheckedDataCount()
{
	return 0;
}

int CtListModelLinkSuggest::getLimitCount()
{
	return 0;
}

void CtListModelLinkSuggest::initialOriginalCheckedIdList()
{
}

void CtListModelLinkSuggest::clearOriginalCheckedIdList()
{
}

bool CtListModelLinkSuggest::checkIsSelectAllOnSearching()
{
	return false;
}

void CtListModelLinkSuggest::initialOriginalPersonDataListSize()
{
}

void CtListModelLinkSuggest::clearOriginalPersonDataListSize()
{
}

int CtListModelLinkSuggest::getOriginalPersonDataListSize()
{
	return 0;
}
