/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _PH_COMMON_H_
#define _PH_COMMON_H_

#include "WDefine.h"
#include <Elementary.h>
#include <string>
#include <app.h>

#define STD_LIST_FREE(target) \
	if(target)\
	{\
		for (auto iter = target->begin(); iter != target->end(); ++iter)\
		{\
			delete *iter;\
		}\
		delete target;\
		target = NULL; \
	}

class WAPP_ASSIST_EXPORT PhCommon
{
public:
	PhCommon();
	virtual ~PhCommon();

public:
	static Evas_Object* createImage(Evas_Object* parent, const char* edj, const char* imagePath, const char* colorCode);
	static Evas_Object* createLayout(Evas_Object* parent, const char* edj, const char* layoutName);
	static Evas_Object* createButton(Evas_Object* parent, const char* buttonStyle, const char* buttonText, Evas_Smart_Cb clickedCb, void* userData );

	typedef enum{
		NORMAL_CALL = 0,
		EMERGENCY_NUMBER,
		EMERGENCY_CONTACTS,
	}emergencyType;

	static void launchCallSetting(Evas_Object *win, Evas_Object *nf);

	/**
	 * @brief Add speed dial number
	 * @param[in]   speedNumber Number on the dialer
	 * @param[in]   numberId    ID of number to assign to @a speedNumber
	 * @return true on success, false if number with this @a numberId already exists.
	 */
	static bool addSpeedDialNumber(int speedNumber, int numberId);

	static void terminateChildAppControl(void);

	static void adjustMatchedLenForName(const char* displayName, const char* keyword, int* index, int* len);
	static int getSavedNumberStartIndex(const char* source, const char* keyword);
	static void adjustMatchedLenForNum(const char* number, const char* keyword, int* index, int* len);
	static std::string convert2MarkupString(const char* source);

private:
	static int __checkUtf8(char c);
	static bool __isDash(char ch);
	static void image_mask_delete_cb(void *data, Evas *e, Evas_Object *obj, void *event_info);
};

#endif /* _PH_COMMON_H_ */

