/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#include "PhDialerKey.h"
#include "ContactsCommon.h"
#include "ContactsDebug.h"
#include "PhDialerLayout.h"

#include <Elementary.h>
#include <feedback.h>

#define DIALER_LAYOUT_EDJ "phone/ph-dialer-keypad.edj"

namespace
{
	char values[] = {
		'1',
		'2',
		'3',
		'4',
		'5',
		'6',
		'7',
		'8',
		'9',
		'*',
		'0',
		'#',
	};

	const char *groups[] = {
		KEYPAD_NUMBER_1,
		KEYPAD_NUMBER_2,
		KEYPAD_NUMBER_3,
		KEYPAD_NUMBER_4,
		KEYPAD_NUMBER_5,
		KEYPAD_NUMBER_6,
		KEYPAD_NUMBER_7,
		KEYPAD_NUMBER_8,
		KEYPAD_NUMBER_9,
		KEYPAD_NUMBER_ASTERISK,
		KEYPAD_NUMBER_0,
		KEYPAD_NUMBER_SHARP
	};

	const feedback_pattern_e patterns[] = {
		FEEDBACK_PATTERN_KEY1,
		FEEDBACK_PATTERN_KEY2,
		FEEDBACK_PATTERN_KEY3,
		FEEDBACK_PATTERN_KEY4,
		FEEDBACK_PATTERN_KEY5,
		FEEDBACK_PATTERN_KEY6,
		FEEDBACK_PATTERN_KEY7,
		FEEDBACK_PATTERN_KEY8,
		FEEDBACK_PATTERN_KEY9,
		FEEDBACK_PATTERN_KEY_STAR,
		FEEDBACK_PATTERN_KEY0,
		FEEDBACK_PATTERN_KEY_SHARP
	};
}

PhDialerKey::PhDialerKey(Id id)
	: m_Id(id), m_Value(values[id]), m_Tone(0)
{
}

Evas_Object *PhDialerKey::onCreate(Evas_Object *parent, void *param)
{
	static const std::string path = ContactsCommon::getAppEdjePath(DIALER_LAYOUT_EDJ);
	Evas_Object *layout = elm_layout_add(parent);
	Eina_Bool res = elm_layout_file_set(layout, path.c_str(), groups[m_Id]);
	WPWARN(res != EINA_TRUE, "elm_layout_file_set() failed");

	evas_object_event_callback_add(layout, EVAS_CALLBACK_MOUSE_DOWN,
			&PhDialerKey::onMouseDown, this);

	return layout;
}

char PhDialerKey::getValue() const
{
	return m_Value;
}

PhDialerKey::Id PhDialerKey::getId() const
{
	return m_Id;
}

void PhDialerKey::onMouseDown(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	PhDialerKey *key = (PhDialerKey*) data;
	feedback_play(patterns[key->m_Id]);
}
