/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _PH_LOG_DB_MGR_H_
#define _PH_LOG_DB_MGR_H_

#include <list>
#include <string>

#include <contacts.h>
////#include <contacts_product.h>

#include "PhDbListener.h"

class PhLogDbMgr
{
public:
	static PhLogDbMgr* getInstance();

public:
	typedef enum {
		LIST_TYPE_ALL_CALL_MESSAGE,
		LIST_TYPE_MISSED_CALL,
		LIST_TYPE_EMAIL,
		LIST_TYPE_ALL_CALL,//more_popup
		LIST_TYPE_DIALED_CALL,
		LIST_TYPE_RECEIVED_CALL,
		LIST_TYPE_REJECTED_CALL,
		LIST_TYPE_ALL_MESSAGE,
		LIST_TYPE_ALL_CALL_MESSAGE_EMAIL,
		LIST_TYPE_SENT_MESSAGE,
		LIST_TYPE_RECEIVED_MESSAGE,
		LIST_TYPE_DETAIL_VIEW,
		LIST_TYPE_VOICE_CALLS,
		LIST_TYPE_VIDEO_CALLS,
		LIST_TYPE_ALL_CALL_SIM_INDEX_1,
		LIST_TYPE_ALL_CALL_SIM_INDEX_2
	}ListType;	//log type

	typedef struct RecordInfo
	{
		RecordInfo();
		~RecordInfo();

		char *displayName;
		char *imgPath;
		char *number;
		char *callDesc;
		char *displayLogTime;

		int logId;
		int personId;
		int numberType;
		int logTime;
		int logType;
		int callDuration;

		std::list<PhLogDbMgr::RecordInfo*> *groupedList;
	}RecordInfo;

	std::list<PhLogDbMgr::RecordInfo*>* loadLogList(ListType listType, bool isUnknownIncluded, int limit = 0);
	std::list<PhLogDbMgr::RecordInfo*>* loadDetailList(const char* searchNumber, const char* searchDesc);

	bool deleteLogList(std::list<RecordInfo*> *logList, int totalCount);
	void dumpLogList(const std::list<RecordInfo*>* list);
	void updateMissedLogDbAsSeen(int index, int type);

	void attach(PhDbListener* listener);
	void detach(PhDbListener* listener);
	static void onNotify(const char* view_uri, void* data);

private:
	PhLogDbMgr(const char* name);
	virtual ~PhLogDbMgr();

	unsigned int __getLogListDataFromDb(contacts_list_h* ctsList, ListType type, int limit);
	unsigned int __getDetailListDataFromDb(contacts_list_h* ctsList, const char* number);

	ListType __requestedListType;
	bool __requestedIsUnknownIncluded;
	std::list<PhDbListener*> __listenerList;
};

#endif //_PH_LOG_DB_MGR_H_

