/*
 * Copyright (c) 2009-2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#ifndef _PH_LOG_GENLIST_HELPER_H_
#define _PH_LOG_GENLIST_HELPER_H_

#include "Elementary.h"
#include "WControl.h"
#include <functional>
#include <vector>

class PhLogGenlistHelper : public WControl
{
public:
	PhLogGenlistHelper(Evas_Object* parent);

public:
	typedef struct {
		Elm_Object_Item* it;
		const void* data;
		const void* p;
		void* contents;
		Ecore_Timer *thumbnailUpdateTimer;
	} ItemData;

	typedef struct {
		Elm_Object_Item* it;
		const void* data;
		const void* p;
		Eina_Bool isChecked;
		Evas_Object *checkButton;
	} CheckedItemData;

	typedef enum{
		SWIPE_TO_RIGHT,
		SWIPE_TO_LEFT
	}SwipeDirection;
public:
	Evas_Object* getEvasObj(){return __genlist;}
	Elm_Object_Item* appendItem(
                        const Elm_Genlist_Item_Class *itc,
                        const void *data,
                        Elm_Object_Item *parent,
                        Elm_Genlist_Item_Type type,
                        Evas_Smart_Cb func,
                        const void *func_data);


	Elm_Object_Item* appendCheckItem(
							const Elm_Genlist_Item_Class *itc,
							const void *data,
							Elm_Object_Item *parent,
							Elm_Genlist_Item_Type type,
							Evas_Smart_Cb func,
							const void *func_data,
							Eina_Bool isChecked = EINA_FALSE);

	void appendFinish();

	void updateItems(void);

	void setSwipeHandler(const std::function<void(SwipeDirection, void*)>& func) {__swipeHandler = func;}

private:
	virtual Evas_Object* onCreate( Evas_Object* parent, void* param );
	virtual void onDestroy();

	static void __swipeStartCb(void *data, Elm_Object_Item *it, const char *em, const char *src);
	static void __swipeInvokeCb(void *data, Elm_Object_Item *it, const char *em, const char *src);
	static void __swipeCancleCb(void *data, Elm_Object_Item *it, const char *em, const char *src);
	static void __genlistRealizedCb(void *data, Evas_Object *obj, void *event_info);
	static void __genlistUnrealizedCb(void *data, Evas_Object *obj, void *event_info);
	static void __languageChangedCb(void *data, Evas_Object *obj, void *event_info);
	static void __longPressedCb(void *data, Evas_Object *obj, void *event_info);
	static Eina_Bool __scrollTimerCb(void *data);
	static void __scrollCb(void *data, Evas_Object *obj, void *event_info);
	static void __mouseUpCb(void *data, Evas *evas, Evas_Object *obj, void *event_info);

	Evas_Object* __createGenlist(void);
private:
	PhLogGenlistHelper();
	virtual ~PhLogGenlistHelper();

private:
	Evas_Object* __genlist;
	Evas_Object* __parent;
	std::function<void(SwipeDirection, void*)> __swipeHandler;
	Ecore_Timer* __scrollTimer;
	bool __isClearRequested;

	int __smartAppendIndex;
};

#endif //_PH_LOG_GENLIST_HELPER_H_

